function check_2mass_settings(conf)
%CHECK_2MASS_SETTINGS
% Helper function for 'convert_all_psc.m'.
% Tests if settings in load_2mass_settings.m (previously loaded in
% parameter "conf") are correct and if all files and programs needed for
% 2MASS star catalog preprocessing are accessible.
% Usage: 
%     conf = load_2mass_settings();
%     check_2mass_settings(conf);
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Default message appended at the end of every error message
endmsg = sprintf(['Set proper paths in "load_2mass_settings.m".\n' ...
    'Read file "01readme.txt" for further instructions.\n']);

% Checks for the existence of the directory where the 2-MASS star catalog
% gzipped sections should be found:
if exist(conf.PSC_src_path, 'dir') == 0
    error('2-MASS star catalog source path "%s" not found!\n%s',...
        conf.PSC_src_path, endmsg);
end

% Checks if each individual section can be found:
%  57 sections in the southern hemisphere of the celestial sphere:
for k=0:56
    filename = [conf.PSC_src_path 'psc_a' num2alfcount(k) '.gz'];
    if exist(filename, 'file') == 0
        error(['File "%s" not found.\n'...
            'Make sure you have downloaded all gzipped sections of '...
            'the 2-MASS star catalog.\n' endmsg], filename);
    end
end
%  35 sections in the northern hemisphere of the celestial sphere:
for k=0:34
    filename = [conf.PSC_src_path 'psc_b' num2alfcount(k) '.gz'];
    if exist(filename, 'file') == 0
        error(['File "%s" not found.\n'...
            'Make sure you have downloaded all gzipped sections of '...
            'the 2-MASS star catalog.\n' endmsg], filename);
    end
end

% Checks the existence of a temporary folder used to store uncompressed
% catalog sections
if exist(conf.PSC_temp_path, 'dir') == 0
    if conf.remove_tmp_files
        middlemsg = 'and deleted (if empty) ';
    else
        middlemsg = '';
    end
    fprintf(['Notice: Folder "%s" not found!\n' ...
        'It will be automatically created ' middlemsg ...
        'during the conversion process.\n' ...
        'Read file "01readme.txt" for details.\n\n'], ...
        conf.PSC_temp_path);
end

% Checks the existence of the folder used to store preprocessed
% catalog sections
if exist(conf.PSC_mat_path, 'dir') == 0
    error(['Folder "%s" not found!\n' ...
        'A place to store preprocessed catalog sections is needed!\n' ...
        endmsg], conf.PSC_mat_path);
end

% Checks the presence of the unzip program
if exist(conf.Unzip7ZipProg, 'file') == 0
    error('Unzip utility %s not found!\n%s', conf.Unzip7ZipProg, endmsg);
end

% Checks the presence of the conversion utility
if exist(conf.convProg, 'file') == 0
    error('Conversion utility "%s" not found!\n%s', conf.convProg, endmsg);
end
