% CONVERT_ALL_PSC
% Function used to convert all sections in the 2MASS catalog to .mat files
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

function convert_all_psc

t_init = tic;

% Read configuration data (paths, program names, etc)
conf = load_2mass_settings;

% Init messages
fprintf('Running script "%s".\n', mfilename);
fprintf('Current working directory is "%s".\n', pwd);

if conf.remove_tmp_files
    disp('Decompressed catalog sections will be removed.');
else
    fprintf('Decompressed catalog sections will be stored in "%s"\n', ...
        conf.PSC_temp_path);
end

% Checking configuration data
check_2mass_settings(conf);

% % % Main processing % % %

% 57 sections in the southern hemisphere of the celestial sphere
for k=0:56
    basename = ['psc_a' num2alfcount(k)];
    fprintf('\nProcessing %s\n', basename);
    do_convert(basename, conf);
end

% 35 sections in the northern hemisphere of the celestial sphere
for k=0:34
    basename = ['psc_b' num2alfcount(k)];
    fprintf('\nProcessing %s\n', basename);
    do_convert(basename, conf);
end


% Remove temporary folder if uncompressed catalog sections should be
% automatically deleted
if conf.remove_tmp_files
    remove_tmp_dir(conf)
end

fprintf(['\n\nElapsed time in catalog preprocessing ' ...
    '(by %s) is %f seconds.\n'], mfilename, toc(t_init));

end



function do_convert(basename, conf)
    ti = tic;
    unpack_cmd = [prepare_fname(conf.Unzip7ZipProg) ...
        ' e ' conf.PSC_src_path basename ...
        '.gz' ' -o' conf.PSC_temp_path];
    convert_cmd = [...
        prepare_fname(conf.convProg)...  % program name
        ' ' basename ... % first argument - catalog name
        ' ' conf.PSC_temp_path basename ... % second argument: source file
        ' ' conf.PSC_mat_path basename '.mat' ... % last arg.: dest. file
        ];
    
    % Run commands
    if run_cmd(unpack_cmd) ~= 0
        error(['Unzip command unsuccessful. ' ...
            'Make sure paths in load_2mass_settings are properly set ' ...
            'and that you have it installed.']);
    end
       
    if run_cmd(convert_cmd) ~= 0
        error(['Convert program failed. ' ...
            'Make sure it exists and paths in load_2mass_settings '...
            'are properly set']);
    end
    
    % If uncompressed catalog sections should be automatically deleted
    if conf.remove_tmp_files
        tmpfile = [conf.PSC_temp_path basename];
        file_recycle_state = recycle();
        delete(tmpfile);
        recycle(file_recycle_state);
        if exist(tmpfile, 'file') ~= 0
            warning(['Unable to delete file %s.\n' ...
                'Manually delete temporary files in folder %s '...
                'to reclaim disk space.'], tmpfile, conf.PSC_temp_path);
        end
    end
    
    toc (ti);
end

function remove_tmp_dir(conf)
% Attempts to remove temporary folder used to store uncompressed catalog
% sections:
if exist(conf.PSC_temp_path, 'dir') ~= 0
    rmdir_cmd = ['rmdir "' conf.PSC_temp_path '"'];
    if run_cmd(rmdir_cmd) ~= 0
        warning(['Unable to remove temporary folder %s.\n' ...
            'Does it have any temporary file inside?\n' ...
            'Delete any temporary files in it manually ' ...
            'to reclaim disk space.'], conf.PSC_temp_path);
    end
end
end

function status = run_cmd(cmd)
    t1 = tic;
    fprintf('%s\n', cmd);
    status = system(cmd);
    fprintf(' => '); toc(t1);
end
