// nosleep.c : Prevents computer entering sleep mode after long periods of 
// no user activity. Useful for keeping the computer awake for performing long
// unassisted tasks.
// By Marcio A. A. Fialho - 2019-04-18 (yyyy-mm-dd)
// This is free software.
// See file readme.txt for more information and licensing.

// Needs at least Windows 2000 or Windows 98  (from mingw32 winbase.h)
#define _WIN32_WINNT 0x0500

#include <windows.h>


// Functions for disabling / reenabling computer/MS-Windows sleep mode.
// https://msdn.microsoft.com/en-us/library/windows/desktop/aa373208(v=vs.85).aspx

// Apparently MINGW-32 header files do not define ES_AWAYMODE_REQUIRED
// so we have to define it ourselves from the Microsoft documentation:
// https://docs.microsoft.com/en-us/windows/desktop/api/winbase/nf-winbase-setthreadexecutionstate
#ifdef __MINGW32__
 #ifndef ES_AWAYMODE_REQUIRED
  #define ES_AWAYMODE_REQUIRED 0x00000040
 #endif
#endif

static int disable_sleep_mode(void)
{
	EXECUTION_STATE previous_EXEC_STATE;

	previous_EXEC_STATE = SetThreadExecutionState(
		ES_AWAYMODE_REQUIRED | ES_SYSTEM_REQUIRED | ES_CONTINUOUS);

	if (previous_EXEC_STATE == 0)
		return -2;

	return 0; // success
}

static int reenable_sleep_mode(void)
{
	EXECUTION_STATE previous_EXEC_STATE;

	previous_EXEC_STATE = SetThreadExecutionState(ES_CONTINUOUS);

	if (previous_EXEC_STATE == 0)
		return -2;

	return 0; // success
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
		LPSTR lpCmdLine, int nCmdShow)
{
	const char* progname = "NoSleep";

	if (disable_sleep_mode() == 0)
	{
		MessageBox(NULL, 
			"Sleep mode disabled.\n"
			"Click OK to reenable Sleep mode.",
			progname, MB_OK);
	}
	else
	{
		MessageBox(NULL,
			"Call to disable_sleep_mode failed - sleep mode not disabled.",
			progname, MB_OK);
	}

	
	if (reenable_sleep_mode() == 0)
	{
		MessageBox(NULL,
			"Sleep mode re-enabled.\n"
			"Click OK to close the Application.",
			progname, MB_OK);
	}
	else
	{
		MessageBox(NULL, 
			"Call to reenable_sleep_mode failed - sleep mode might still be disabled.\n"
			"Click OK to close Application.",
			progname, MB_OK);
	}

	return 0;
}