function fluxref = VegaRefmag(band, Vegamag)
% fluxref = VegaRefmag(band, Vegamag)
% Computes reference fluxes in passband 'band' using star Vega as a 
% reference.
%
% Inputs:
%  band - An m x 2 matrix with the band energy response (responsivity)
%         for the spectral band considered. 
%         First column is the wavelength in meters;
%         Second column is the responsivity (in units of A/W, lumens/W, or
%         in any other unit, or normalized so that the maximum is set to
%         one).
%  Vegamag - Assigned Vega (Alpha Lyr) magnitude in that band => This
%         defines the zero point of this magnitude scale.
%
% Output:
%  Flux that defines the zero point of the magnitude scale based on
%  band 'band', in A/m^2, lumens/m^2 or W/m^2 when the band response
%  specified in the second column of 'band' is given in A/W, lumens/W or
%  normalized to unity, respectively.
%
% Copyright (C) 2016-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Loads and processes spectral data for Vega (alpha Lyr:)
% File alpha_lyr_stis_008.fits downloaded on 2016-11-11 (yyyy-mm-dd) from
% http://www.stsci.edu/hst/observatory/crds/calspec.html (old website)
% This file is now available at: ftp://ftp.stsci.edu/cdbs/calspec/
% See file "calspec/readme.txt" for more information.

vegaFileName = 'calspec/alpha_lyr_stis_008.fits';
vega = read_calspec(vegaFileName);
vega_lambda = vega{1} * 1e-10;
vega_flux = vega{2} * 1e7;
 % multiplication by 1e-10 to convert from Angstroms to meters.
 % multiplication by 1e7 to convert from erg/s/cm^2/angstrom to W/m^2/m

% Checks spectral band limits against data loaded from .fits file.
if ( min(vega_lambda) > min(band(:,1)) ) || ...
   ( max(vega_lambda) < max(band(:,1)) )
    warning(['Data in "%s" does not cover the spectral band entirely.\n'...
        'Results may be inaccurate.'], vegaFileName);
end

% small differences used for numerical integration.
vega_lambdadif = filter([1 -1], 1, vega_lambda);
vega_lambdadif(1) = 0;

% Computes Vega's flux in band 'band'
B = interp1(band(:,1), band(:,2), vega_lambda, 'pchip', 0);
vega_fluxint = filter(1, [1 -1], vega_flux .* vega_lambdadif .* B);
vega_Bflux = vega_fluxint(end);

% Computes reference flux
fluxref = vega_Bflux * 10^(0.4 * Vegamag);

end