% Script used to compute values shown in Table 2 of the "Theoretical
% Limits of Star Sensor Accuracy" journal article.
%
% Notice: 
%         To run this script in a reasonable time, without too much disk
%         swapping, at least 8 GB of RAM are needed. A similar script, that
%         does not require too much memory to run and which is based on the
%         Hipparcos star catalog is calcTable2_Hip.m.
%         This script runs in about 90 seconds in a desktop machine
%         equipped with 8GB of RAM and an SSD, when run in Matlab.
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

clear;
close all;

% Configuration Parameters
params.diam = 0.1; %0.1; % Star sensor diameter in meters
params.texposure = 0.01; % 0.1; % Exposure time in seconds
params.Tmin = 2000;  % Minimum black-body temperature in kelvins.
params.Tmax = 30000; % Maximum black-body temperature in kelvins.
params.TinvStep = 4e-7; % inverse of temperature step when
% generating a list of black bodies with temperatures from Tmin to Tmax.
params.log10integStep = 0.0001; % base-10 logarithm of integration
% step in wavelength.
params.TwoMASSJHK01_matfile = 'c:\2mass\data\psc_JHK01.mat';
% preprocessed star catalog => First section of the 2MASS star catalog,
% containing 26,036,431 brightest stars in J band.
params.FOVlistd = [360, 90, 25, 10, 4]; % FOV diameters in degrees
% params.FOVlistd = [360, 180, 90, 45, 22.5, 11.25, 5.625, 2.8125]; % FOV diameters in degrees
% params.FOVlistd = [360, 180, 40, 20, 12, 8, 4, 3, 2, 1, 0];
params.optical_heads = [... % List of optical heads pointing directions
    % first column = right ascension in degrees
    % second column = declination in degrees    
    %
    %Canonical optical axis directions:
     0,  0; ...   % first direction:  R.A. = 0 deg, dec = 0 deg.
     90, 0; ...   % second direction: R.A. = 90 deg, dec = 0 deg.
    % 0, 90; ...   % third direction:  R.A. = 90 deg, dec = 0 deg.
    %
    %Other directions:
    % 45, 45; ...
    % 135,-45; ...
    % 186.65, -63.1     % near star Acrux in the Southern Cross. A star
                        % rich region in the sky.                        
    % -1.690, -26.6550  % Towards a region with few stars that is almost
                        % perperdicular to the direction of Acrux.
    % 84,  -1   % near the center of Orion's Belt.
    % -6,   0   % perpendicular to previous direction.
    ];

params.sensor_combinations = [... % List of star sensor combinations
    % Number of columns must be equal to number of star sensors optical
    % heads in the list above. Each row is a combination of star sensors to
    % be used together. Put 1 to indicate presence of optical head and 0 to
    % indicate it is not present:
    1, 0; ... % only optical head 1 present
    0, 1; ... % only optical head 2 present
    1, 1; ... % optical heads 1 and 2 present
    ];

t1 = tic;

fprintf('Running calcTable2 with the following parameters:\n');
disp(params);

% If running GNU-Octave, sets default format to use in .mat files
setup_octave_savemat();

if exist(params.TwoMASSJHK01_matfile, 'file') == false
    error(['Preprocessed star catalog (file %s) not found!\n' ...
        'Run calc_2MASS.m to generate it.'], params.TwoMASSJHK01_matfile);
else
    fprintf(['Loading first section of preprocessed 2MASS ' ...
        'catalog (file %s).\n'], params.TwoMASSJHK01_matfile);
    load(params.TwoMASSJHK01_matfile);
    % Description of fields of psc_JHK01 in the .mat file loaded above
    % (see lower_bound_from_starcat.m, lines 549-553)
    % psc_JHK01(:,1) == star R.A in degrees
    % psc_JHK01(:,2) == star dec. in degrees
    % psc_JHK01(:,3) == 2MASS J magnitude
    % psc_JHK01(:,4) == 2MASS H magnitude
    % psc_JHK01(:,5) == 2MASS Ks magnitude
end


% loads Physical and Mathematical constants
consts = loadConsts(false);

% Load spectral bands
[bList, Vegamag] = loadBands(); % run function in loadBands.m

% Spectral band configurations  (adapted from lower_bound_from_starcat.m)
band_order = 1; % J band is at shorter wavelengths than H band
%  The primary spectral band is the J passband.
b1.name = 'J';
b1.data = bList.J;
b1.Vegamag = Vegamag.J;
%  The secondary spectral band is the H passband.
b2.name = 'H';
b2.data = bList.H;
b2.Vegamag = Vegamag.H;

% Compute reference fluxes for bands b1 and b2
b1.Ref = VegaRefmag(b1.data, b1.Vegamag);
b2.Ref = VegaRefmag(b2.data, b2.Vegamag);

fprintf('Reference flux for band %s = %e W/m^2\n', b1.name, b1.Ref);
fprintf('Reference flux for band %s = %e W/m^2\n', b2.name, b2.Ref);

% Compute black-body magnitudes. Also returns in struct BB temperatures
% and wavelength tables for interpolation and numerical integration.
% BB is an acronym for black-body.
BB = calc_BlackBody_mags(params, consts, b1, b2, band_order, true);

% Filtering all stars with V magnitude above max_magV
catsize = size(psc_JHK01, 1);
fprintf('Read %d stars from section 1/20 in 2MASS\n', catsize);

disp('Computing equiv. black-bodies for all stars in the catalog.');
J_mags = psc_JHK01(:,3);
H_mags = psc_JHK01(:,4);
[Ti, Ci, nclampTop, nclampBot] = calc_equiv_black_bodies(...
    J_mags, H_mags, BB, b2.name, true);
nclamp = nclampTop + nclampBot;

% Computes the lower bound on centroiding error for each star
% cat_F is the reciprocal of the variance lower bound = 1/sigma_min^2
disp('Computing lower bound on centroiding accuracy for all stars.');
InfoFactor = consts.InfoFactor * params.diam^4 * params.texposure;
cat_F = InfoFactor  .* Ci .* (Ti .^ 5);

% Computes unit vectors for every star in the star subcatalog
% Notice: angles in the star subcatalog in degrees.
disp('Computing unit vector representation for all stars.');
catvect = zeros(catsize, 3);
catvect(:,1) = cosd(psc_JHK01(:, 1)) .* cosd(psc_JHK01(:, 2)); % X component
catvect(:,2) = sind(psc_JHK01(:, 1)) .* cosd(psc_JHK01(:, 2)); % Y component
catvect(:,3) = sind(psc_JHK01(:, 2)); % Z component of unit vector

clear('psc_JHK01');

% Sorts star catalog by increasing values of the Fisher information.
% By summing first the smaller values we minimize roundoff errors.
catvect(:,4) = cat_F;
catvect(:,5) = 1:catsize; % indexes for debugging purposes
cat2 = sortrows(catvect, 4);

clear('catvect');

% Computes the pointing directions of the star sensors in cartesian
% coordinates:
opax = params.optical_heads;
numsens = size(opax, 1);
sensvect = zeros(numsens, 3);
sensvect(:,1) = cosd(opax(:, 1)) .* cosd(opax(:, 2)); % X component
sensvect(:,2) = sind(opax(:, 1)) .* cosd(opax(:, 2)); % Y component
sensvect(:,3) = sind(opax(:, 2)); % Z component of unit vector

% Computes dot products between stars and star sensors directions
cat3 = zeros(catsize, numsens);
for k=1:numsens
    cat3(:,k) = dot(cat2(:,1:3)', repmat(sensvect(k,:),catsize,1)')';
end

% For the whole celestial sphere:
%  Computes information matrix F
F = calcFisherMat(cat2(:,1:3)', cat2(:,4)');

%  Computes the covariance matrix - equation 5.113 in Ref. 1
disp('Computing lower bound on attitude accuracy:');
Pthetatheta = inv(F);

%  Computes the variance on the attitude error
var_theta = trace(Pthetatheta);

%  Computes and prints the attitude error standard deviation
%  See section 2.11 in the journal article.
sigma_theta = sqrt(var_theta);
fprintf('theta_rms = %.16g rad = %.16g mas\n', sigma_theta,...
    sigma_theta * 180/pi * 3600 * 1000);


% Now for optical heads with restricted fields of view.
FOVlist = params.FOVlistd;
numfovs = size(FOVlist, 2);
comblist = params.sensor_combinations;
numcombs = size(comblist, 1);
comb_att_err_list = zeros(numfovs, numcombs);

for m=1:numfovs
    fprintf('\nFor FOVs of %d degrees:\n', FOVlist(m));
    fovradius = 0.5 * FOVlist(m); % FOV radius in degrees
    cosrfov = cosd(fovradius);
    fprintf(' FOV radius = %g deg.\n', acosd(cosrfov)); % verification!
    
    % For each optical head defined in FOVlist:
    for k=1:numsens
        % Selects all stars within a conical FOV with a radius of fovradius
        % degrees:
        indx = (cat3(:, k) >= cosrfov);
        catr = cat2(indx, :);
        nstars = size(catr, 1);
        fprintf(' For star sensor %d: %d stars in FOV.\n', k, nstars);
        
        if (nstars >= 2)
            F = calcFisherMat(catr(:, 1:3)', catr(:, 4)');
            Pthetatheta = inv(F);
            var_theta = trace(Pthetatheta);
            sigma_theta = sqrt(var_theta);
            fprintf('   theta_rms = %.16g rad = %.16g arcsec\n',...
                sigma_theta, sigma_theta * 180/pi * 3600);
        end
    end
    
    if numcombs > 0
        fprintf('\n');
    end
    
    % For each combination of optical heads defined in comblist:
    for cn = 1:numcombs
        numheads = 0;
        catr = [];
        cath_list_msg = '';
        for k=1:numsens
            if comblist(cn, k) == 1
                numheads = numheads + 1;
                % Selects all stars within a conical FOV with a radius of
                % fovradius degrees for optical head k
                indx = (cat3(:, k) >= cosrfov);
                catrk = cat2(indx, :);
                if numheads == 1
                    catr = catrk;
                    cath_list_msg = sprintf('%d', k);
                else
                    catr = [catr; catrk]; %#ok<AGROW>
                    cath_list_msg = ...
                        [cath_list_msg sprintf(', %d', k)]; %#ok<AGROW>
                end
            elseif comblist(cn, k) ~= 0
                error(['params.sensor_combinations(%d, %d) '...
                    'must be either 0 or 1.'], cn, k);
            end
        end
        
        if numheads == 0
            warning([' No optical head selected in row %d in '...
                'params.sensor_combinations'], cn);
        elseif numheads >= 1
            nstars = size(catr, 1);
            if numheads == 1
                fprintf([...
                    ' For single optical head'...
                    ' %s with %d stars:\n'], cath_list_msg, nstars);
            else
                fprintf([...
                    ' For combination of optical heads'...
                    ' [%s] with %d stars:\n'], cath_list_msg, nstars);
            end
            
            if (nstars >= 2)
                F = calcFisherMat(catr(:, 1:3)', catr(:, 4)');
                Pthetatheta = inv(F);
                var_theta = trace(Pthetatheta);
                sigma_theta = sqrt(var_theta);
                fprintf('   theta_rms = %.16g rad = %.16g arcsec\n',...
                    sigma_theta, sigma_theta * 180/pi * 3600);
                comb_att_err_list(m, cn) = sigma_theta;
            else
                % NaN used as a marker to indicate non-computed results:
                comb_att_err_list(m, cn) = nan;
            end
            
            % Debug
            if fovradius == 1  % one degree
                disp(Pthetatheta);
            end
            
        else
            error('Internal error: invalid numheads = %f', numheads);
        end
    end
end


% Prints a table with results for combinations of optical heads:
fprintf('\n');
fprintf('Consolidated list: theta_rms in radians:\n');
fprintf('FOV(deg)'); % header line
fmt_prec_spec = '%.6g'; % precision specifier, use %.18g for full accuracy.
fmt_string = '%4g'; % format string for a later call to fprintf
for cn = 1:numcombs
    fprintf('\tcomb%d', cn);
    fmt_string = [fmt_string '\t' fmt_prec_spec]; %#ok<AGROW>        
end
fprintf('\n'); % complete header line
fmt_string = [fmt_string '\n'];
fprintf(fmt_string, [FOVlist', comb_att_err_list]') % print table contents
fprintf('\n');

fprintf('Consolidated list: theta_rms in milli-arc-seconds:\n');
arcsec = (pi/180)/3600; % One arc-second in radians.
fprintf('FOV(deg)'); % header line
fmt_prec_spec = '%.4f'; % precision specifier, use %.18g for full accuracy.
fmt_string = '%4g'; % format string for a later call to fprintf
for cn = 1:numcombs
    fprintf('\tcomb%d', cn);
    fmt_string = [fmt_string '\t' fmt_prec_spec]; %#ok<AGROW>        
end
fprintf('\n'); % complete header line
fmt_string = [fmt_string '\n'];
% Print table contents:
fprintf(fmt_string, [FOVlist', comb_att_err_list * 1000/arcsec]');
fprintf('\n');


dt1 = toc(t1);
fprintf('Script %s took %f seconds.\n', mfilename, dt1);
disp('Done.');
