function [cat_Ti, cat_Ci, nclampTop, nclampBot] = ...
    calc_equiv_black_bodies(...
    starcatmag1, starcatmag2, BB, b2name, verbose)
% [cat_Ti, cat_Ci, nclampTop, nclampBot] = ...
%    calc_equiv_black_bodies(...
%    starcatmag1, starcatmag2, BB, b2name, verbose)
% Computes the equivalent black-body models for each star in the star
% catalog.
% inputs:
%   starcatmag1 = cataloged star magnitudes in band 1.
%   starcatmag2 = cataloged star magnitudes in band 2.
%   BB = struct containing curve relating color indices with temperature
%        generated by calc_BlackBody_mags.m
%   b2name = name of the secondary band.
%   verbose = verbosity flag => If true, detailed text is printed.
% outputs:
%   Ti = temperatures of the black-body equivalents for each star.
%   Ci = dilution factor for the black-body equivalents.
%   nclampTop = number of stars that had their magnitudes in band 2
%               adjusted so that their color index were equal to the
%               hottest black-body in BB.
%   nclampBot = number of stars that had their magnitudes in band 2
%               adjusted so that their color index were equal to the
%               coldest black-body in BB.
%
% Limitations: 
%  1 -  The curve relating the color indices (BB.color) with
%       reciprocal of temperature (BB.Tinv) must be invertible 
%       (strictly increasing or strictly decreasing).
%  2 - The field BB.Tinv in BB must be strictly increasing.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% BB.Tinv should be strictly increasing, and should not contain NaN's
% (IEEE Not a Number)
assert(sum(isnan(BB.Tinv))==0);
Tinv_diff = diff(BB.Tinv);
assert(min(Tinv_diff) > 0);

% Checking if BB.color is strictly increasing or strictly decreasing
assert(sum(isnan(BB.color)) == 0);
color_diff = diff(BB.color);
if ~( min(color_diff)*max(color_diff) > 0)
    error(['The computed %s color index is not an invertible function' ...
        ' of temperature in the range %f K < T < %f K.' ...
        ' This algorithm does not work in such situations.'], ...
        BB.colorIdxName, BB.T(end), BB.T(1));
end

% Computes the color index for all stars in the catalog:
if BB.band_order > 0
    colorIdx = starcatmag1 - starcatmag2; % band1 - band2
else
    colorIdx = starcatmag2 - starcatmag1; % band2 - band1
end

% Counts the number of stars that will have their band2 magnitudes adjusted
[minColorIdx, ordering]  = min([BB.color(1), BB.color(end)]);
[maxColorIdx, ordering2] = max([BB.color(1), BB.color(end)]);
assert (ordering + ordering2 == 3);
nclampTop = sum(colorIdx < minColorIdx);
nclampBot = sum(colorIdx > maxColorIdx);

if verbose
    if (ordering == 1)
    fprintf( ['%d stars with %s < %f (Teq > %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        nclampTop, BB.colorIdxName, BB.color(1), BB.T(1), ...
        BB.colorIdxName, BB.color(1), b2name);
    fprintf( ['%d stars with %s > %f (Teq < %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        nclampBot, BB.colorIdxName, BB.color(end), BB.T(end), ...
        BB.colorIdxName, BB.color(end), b2name);
    elseif (ordering == 2)
    fprintf( ['%d stars with %s < %f (Teq < %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        nclampTop, BB.colorIdxName, BB.color(end), BB.T(end), ...
        BB.colorIdxName, BB.color(end), b2name);
    fprintf( ['%d stars with %s > %f (Teq > %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        nclampBot, BB.colorIdxName, BB.color(1), BB.T(1), ...
        BB.colorIdxName, BB.color(1), b2name);
    else
        error('Internal consistency error.');
    end
end

% Clamps color indexes.
colorIdxClamped = min(max(colorIdx, minColorIdx), maxColorIdx);

% From that, compute the equivalent black-body temperatures for all 
% stars, by interpolating over the relation between color index and
% inverse of temperature for black-bodies
cat_Tinv = interp1(BB.color, BB.Tinv, colorIdxClamped, 'pchip', 0);
cat_Ti = 1./ cat_Tinv;

% Computing the apparent magnitude in band1 that the equivalent 
% black-body that represents each star would have for an observer located
% on its surface. cat_BB_b1mag will be typically between -30 and -45
cat_BB_b1mag = interp1(BB.Tinv, BB.b1mag, cat_Tinv, 'pchip', 0); 

% Compute dilution factor for all stars:
cat_Ci = 10 .^ (0.4 * (cat_BB_b1mag - starcatmag1));

end