function consts = loadConsts(verbose)
% consts = loadConsts(verbose)
% Definitions of some fundamental and derived constants.
% If input parameter verbose is set to true, some informative text is
% also printed.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Mathematical constants
%   zeta(n) = Riemann-Zeta function evaluated at n.
consts.zeta5 = 1.036927755143369926331; % OEIS sequence A013663
consts.zeta4 = pi^4/90;
consts.zeta3 = 1.2020569031595942853997; % OEIS sequence A002117

% Universal constants from 
% https://www.bipm.org/utils/common/pdf/si-brochure/SI-Brochure-9.pdf
% (updated on May 24, 2019, following SI redefinition in 2019)
consts.c = 299792458; % speed of light in m/s
consts.h = 6.62607015e-34; % Planck constant in J.s
consts.k = 1.380649e-23; % Boltzmann constant in J/K

% Stefan-Boltzmann constant
%   Sources: 
%     * https://physics.nist.gov/cgi-bin/cuu/Value?eqsigma 
%         (click over the symbol of the constant to see the equation);
%     * the definition of hbar constant: hbar = h/(2*pi);
%     * and the definition of consts.zeta4 above.
%   See also : https://en.wikipedia.org/wiki/Stefan%E2%80%93Boltzmann_law
consts.sigmaSB = 12 * pi * consts.zeta4 * ...
    consts.k^4 / consts.c^2 / consts.h^3;

% Equivalent of 'Stefan-Boltzmann' constant for number of photons
consts.tauSB = 4 * pi * consts.zeta3 * ...
    consts.k^3 / consts.c^2 / consts.h^3;

if verbose
    fprintf('Derived constants:\n');
    fprintf(' Stefan Boltzmann constant = %.8g W/m^2/K^4\n',...
        consts.sigmaSB);
    fprintf(' equiv. for number of photons = %.8g photons/s/m^2/K^3\n',...
        consts.tauSB);
end

% Constants in equation that computes the upper bound on the inverse of
% the variance of centroiding error for a point source having the same
% flux of a black-body with temperature T and dilution factor C
% (equation 19 in the paper)
% Equation (19) is  1/sigma_{min}^2 = ctrInfoFactor * D^4 * t * C * T^5
consts.A = 12 * consts.zeta5 * pi^4;
consts.B = (consts.k / consts.h)^5 / consts.c^4;
consts.InfoFactor = consts.A * consts.B;

end