function outputs = lower_bound_from_2mass(...
    catOpt, input_params, verbose)
% Computes an estimate of the lower bound on star tracker accuracy, using
% data from the 2MASS star catalog and the black-body model for stars
% described in the "Theoretical Limits of Star Sensor Accuracy" journal
% article.
%
% Syntax:
%   outputs = STR_Lower_bound_from_2mass(catOpt)
%   outputs = STR_Lower_bound_from_2mass(catOpt, input_params)
%   outputs = STR_Lower_bound_from_2mass(catOpt, input_params, verbose)
%
% Inputs:
%  catOpt: It can be one of the following:
%    'JH' = Uses J and H magnitudes from the 2MASS star catalog.
%           Sorts star catalog by J, H, Ks, RA, dec., before computing
%           outputs.
%    'JK' = Using J and Ks magnitudes from the 2MASS star catalog.
%           Sorts star catalog by J, Ks, H, RA, dec., before computing
%           outputs.
%    'HK' = Using H and Ks magnitudes from 2MASS star catalog.
%           Sorts star catalog by H, Ks, J, RA, dec., before computing
%           outputs.
%
%  input_params: A struct with configuration data for this function.
%            If it is not specified, or if there are missing fields,
%            defaults are used.
%   Description of fields in input_params:
%    diam = ideal spherical star tracker diameter in meters. (default = 1)
%    texposure = exposure time in seconds. (default = 1)
%    Tmin = minimum temperature in kelvins for equivalent black-bodies
%           that represent a star from the catalog. (default = 2000)
%    Tmax = maximum temperature in kelvins for equivalent black-bodies
%           that represent a star from the catalog. (default = 30000)
%
%  verbose: if true, detailed diagnostic messages are printed during
%           execution. (default = false)
%
% Output:
%  outputs: a structure containing data generated by this function.
%
% Warning: This function takes about 2 hours to run!
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% References:
%
% Ref1 - F. L. Markley, J. L. Crassidis. Fundamentals of Spacecraft
%        Attitude Determination and Control. Springer. New York, 2014.
%        ISBN 978-1-4939-0801-1.


t1 = tic;

% Default configurations
% Can be overriden by specifying equivalent field names in input_params.
default_params = load_default_params();

% Processing input parameters.
if nargin < 2
    params = default_params;
else
    params = merge_input_params(input_params, default_params);
end

if nargin < 3
    verbose = false;
end

params.cat_option = catOpt; % saves some input arguments as parameters
params.verbose = verbose; % saves some input arguments as parameters

if verbose
    print_params(params);
end


% Copy some internal variables to the output, for debugging purposes.
if nargin >= 2
    outputs.input_params = input_params;
end

outputs.default_params = default_params;
outputs.params = params;
clear input_params;
clear default_params;

% Checks program parameters for consistency
check_params(params);

% loads Physical and Mathematical constants
consts = loadConsts(verbose);

% Select spectral bands to use, depending on the selected catalog.
[b1, b2, band_order, sort_order, plotStyle] = ...
    loadCatBands(catOpt, verbose);

% Compute reference fluxes for bands b1 and b2
b1.Ref = VegaRefmag(b1.data, b1.Vegamag);
b2.Ref = VegaRefmag(b2.data, b2.Vegamag);

if verbose
    fprintf('Reference flux for band %s = %e W/m^2\n', b1.name, b1.Ref);
    fprintf('Reference flux for band %s = %e W/m^2\n', b2.name, b2.Ref);
end

% Compute black-body magnitudes. Also returns in struct BB temperatures
% and wavelength tables for interpolation and numerical integration.
% BB is an acronym for black-body.
BB = calc_BlackBody_mags(params, consts, b1, b2, band_order, verbose);

if params.plotBands
    % Plot bands spectral responses
    plotBands(BB, b1.name, b2.name, plotStyle);
end

% Checks if star catalog is complete and if all sections are correctly
% sorted in column col_number.
fprintf('Checking catalog integrity:\n');
col_number = sort_order(1);
check_catalog_sections(params, col_number);
Nlist = adjust_Nlist(params.Nlist, params.catsize);

% Loads and process star catalog, section by section.
numsections = params.numsections; % number of sections in the processed
                                  % 2-MASS star catalog.
fprintf('\nComputing the Fisher information matrix for %d sections:\n',...
    numsections);
time_Fisher = tic;

assert(Nlist(end) == params.catsize);
Nlistsize = size(Nlist, 2);
F = zeros(3); % cumulative Fisher information matrix
Fbin = zeros(3); % Fisher information matrix for stars in a bin.
FbinList = zeros(3, 3, Nlistsize);
FsumList = zeros(3, 3, Nlistsize);
Nlistpos = 1;
nclamp = 0;
nclampvect = zeros(numsections);
catsectsizeList = zeros(numsections);
numProcStars = 0; % number of processed stars from previous sections
for numsect = 1:numsections
    
    fprintf('Loading and sorting 2MASS section %d/%d ',...
        numsect, numsections);
    starcat = load_CatalogSection(params, numsect, sort_order, true);
        
    fprintf(' Computing equiv. black-bodies for stars in section %d.\n',...
        numsect);
    [Ti, Ci, nclampsect] = calc_equiv_blackbodies(...
        starcat, BB, b2.name, verbose);
    nclampvect(numsect) = nclamp;
    nclamp = nclamp + nclampsect;
    
    catsectsize = size(starcat, 1); % catalog length
    catsectsizeList(numsect) = catsectsize;
    
    % Computes the lower bound on centroiding error for each star
    % cat_F is the reciprocal of the variance lower bound = 1/sigma_min^2
    disp(' Computing lower bound on centroiding accuracy.');
    InfoFactor = consts.InfoFactor * params.diam^4 * params.texposure;
    cat_F = InfoFactor  .* Ci .* (Ti .^ 5);
    
    % Computes unit vectors for every star in the star catalog
    disp(' Computing unit vector representation for all stars in section.');
    catvect = zeros(catsectsize, 3);
    catvect(:,1) = cos(starcat(:, 1)) .* cos(starcat(:, 2)); % X component
    catvect(:,2) = sin(starcat(:, 1)) .* cos(starcat(:, 2)); % Y component
    catvect(:,3) = sin(starcat(:, 2)); % Z component of unit vector
    
    clear starcat;    
    % computes the contribution to Fisher information matrix from this
    % catalog section.
    disp(' Computing section contribution to Fisher information matrix.');
    for s=1:catsectsize
        Fbin = Fbin + (eye(3) - catvect(s,:)' * catvect(s,:)) * cat_F(s);
        if (s + numProcStars) == Nlist(Nlistpos)
            F = F + Fbin;
            FbinList(:,:,Nlistpos) = Fbin;
            FsumList(:,:,Nlistpos) = F;
            Fbin = zeros(3);
            Nlistpos = Nlistpos+1;
        end
    end
    clear Ti;
    clear Ci;
    clear catvect;    
    numProcStars = numProcStars + s;
end
fprintf('Fisher matrix computations took %f seconds.\n', toc(time_Fisher));

% Sanity check
if numProcStars ~= params.catsize
    warning (['numProcStars = %d and params.catsize = %d differs!!!\n' ...
        'Results may be unreliable!!!'], numProcStars, params.catsize);
end

% Computes the covariance matrix - equation 5.113 in Ref. 1
disp('Computing lower bound on attitude accuracy:');
Pthetatheta = inv(F);

% Computes the variance on the attitude error
var_theta = trace(Pthetatheta);

% Computes and prints the attitude error standard deviation
sigma_theta = sqrt(var_theta);

fprintf('sigma_attitude = %.16g rad = %.16g mas\n', sigma_theta,...
    sigma_theta * 180/pi * 3600 * 1000);

% Plotting some results.
if params.plotFmat
    plotFmat(FsumList, Nlist, params.catname, b1.name, b2.name);
end

if params.plotCovmat
    plotInvCovMat(FsumList, Nlist, params.catname, b1.name, b2.name);
end

% Pass some internal variables to the output.
if nargout > 0
    disp('Copying some internal variables to the output.');
end
outputs.consts = consts;
outputs.b1 = b1;
outputs.b2 = b2;
outputs.plotStyle = plotStyle;
outputs.BB = BB;
outputs.nclamp = nclamp;
outputs.Nlist = Nlist;
outputs.FbinList = FbinList;
outputs.FsumList = FsumList;
outputs.F = F;
outputs.nclampvect = nclampvect;
outputs.nclamp = nclamp;
outputs.catsectsizeList = catsectsizeList;
outputs.numstars = numProcStars;

fprintf('Elapsed time in %s = %f seconds.\n', mfilename, toc(t1));

end



%%%%%%%%%%%%%%%%%%%%%% Functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function default_params = load_default_params
% Default configurations
default_params.diam = 1;  % STR diameter in meters
default_params.texposure = 1; % STR exposure time in seconds
default_params.Tmin = 2000;  % Minimum black-body temperature in kelvins.
default_params.Tmax = 30000; % Maximum black-body temperature in kelvins.
default_params.TinvStep = 4e-7; % inverse of temperature step when 
  % generating a list of black bodies with temperatures from Tmin to Tmax.
default_params.log10integStep = 0.0001; % base-10 logarithm of integration
  % step in wavelength.
default_params.Nlist = [2, 10, 100, 1000, 10000, 100000, 1000000,...
    10000000, 100000000, 200000000, 300000000, 400000000,...
    500000000, 1000000000, 10000000000];
default_params.catname = '2MASS';
default_params.TwoMASSJHK_basename = '2mass/data/psc_JHK';
  % Notice: the same sections used for the 2MASS catalog sorted by J,H,Ks
  % magnitudes can be used for the 2MASS catalog sorted by J,Ks,H
  % magnitudes, just by resorting them by J, Ks and H magnitudes.
default_params.TwoMASSHKJ_basename = '2mass/data/psc_HKJ';
default_params.numsections = 20; % files psc_JHK01.mat to psc_JHK20.mat and
                                 % files psc_HKJ01.mat to psc_HKJ20.mat.
default_params.catsize = 470991651; % number of entries in 2MASS PSC (point
  % source catalog), after rejecting some entries with missing fields.
default_params.cat_basename_JHK = 'psc_JHK'; % base name for the matrix
  % inside each catalog section containing stars in that section.
  % These matrices are named psc_JHK01 for psc_JHK01.mat, psc_JHK02 for
  % psc_JHK02.mat and so on until psc_JHK20 for psc_JHK20.mat.
default_params.cat_basename_HKJ = 'psc_HKJ'; % the same for catalog ordered
  % by H, Ks, J magnitudes, in that order.
default_params.plotBands = false;
default_params.plotFmat = false;
default_params.plotCovmat = true;
end



function check_params(params)
% Checks program parameters for consistency

% Checking if params.Nlist is monotonically increasing
Nlistdiff = diff(params.Nlist);
if min(Nlistdiff) < 1
    error('params.Nlist must be a strictly increasing vector of integers');
end

end



function print_params(params)

disp(['Running function lower_bound_from_2mass with the following '...
    'parameters:']);

fnames = fieldnames(params);
formatString = sprintf(' params.%%-%ds = ', getLongestStringLen(fnames));
N = size(fnames, 1);
for k=1:N
    fieldname = fnames{k};
    fprintf(formatString, fieldname);
    disp(params.(fieldname));
end
end



function [maxlen, N] = getLongestStringLen(textlist)
% [maxlen, N] = getLongestStringLen(textlist)
% Returns the length of the longest string in a list
% maxlen is the length of the longest string.
% N is the number of strings in that list.

if size(textlist, 2) ~= 1  ||  ~ismatrix(textlist)
    error('Cannot handle multidimensional text lists');
end
N = size(textlist, 1);
maxlen = 0;
for k=1:N
    if size(textlist{k}, 1) ~= 1  ||  ~ismatrix(textlist{k})
        error('Each element in the text list must be a row vector');
    end
    maxlen = max(maxlen, size(textlist{k}, 2));
end
end



function [b1, b2, band_order, sort_order, plotStyle] = ...
    loadCatBands(catOpt, verbose)
% Select spectral bands according to star catalog option.
%  b1 is the primary spectral band.
%  b2 is the secondary spectral band.
%  band_order == +1 if the first band is at a shorter wavelength than
%                   band 2, and -1 otherwise.
%  plotStyle is the plot style for spectral bands b1 and b2

[bList, Vegamag] = loadBands(); % run function in loadBands.m

if verbose
    fprintf('Loading spectral band for option %s\n', catOpt);
end

if strcmp(catOpt, 'JH') == true
    band_order = 1;
    sort_order = [3 4 5 1 2]; % Will sort by J, H, Ks, RA, dec.
    
    b1.name = 'J';
    b1.data = bList.J;
    b1.Vegamag = Vegamag.J;
    plotStyle.b1Color = 'b'; % plot this infrared band in blue
    
    b2.name = 'H';
    b2.data = bList.H;
    b2.Vegamag = Vegamag.H;
    plotStyle.b2Color = 'k'; % plot this band in black

elseif strcmp(catOpt, 'JK') == true
    band_order = 1;
    sort_order = [3 5 4 1 2]; % Will sort by J, Ks, H, RA, dec.
    
    b1.name = 'J';
    b1.data = bList.J;
    b1.Vegamag = Vegamag.J;
    plotStyle.b1Color = 'b'; % plot this infrared band in blue
    
    b2.name = 'K';
    b2.data = bList.K;
    b2.Vegamag = Vegamag.K;
    plotStyle.b2Color = 'r'; % plot this band in red

elseif strcmp(catOpt, 'HK') == true
    band_order = 1;
    sort_order = [4 5 3 1 2]; % Will sort by H, Ks, J, RA, dec.
    
    b1.name = 'H';
    b1.data = bList.H;
    b1.Vegamag = Vegamag.H;
    plotStyle.b1Color = 'k'; % plot this infrared band in black
    
    b2.name = 'K';
    b2.data = bList.K;
    b2.Vegamag = Vegamag.K;
    plotStyle.b2Color = 'r'; % plot this band in red
    
else
    error('Unrecognised catalog band option ''%s''', catOpt);
end

if verbose
    fprintf(' Primary passband:\n  ');
    disp(b1);
    fprintf(' Secondary passband:\n  ');
    disp(b2);
end

end



function check_catalog_sections(params, col_number)
% Checks if star catalog is complete and if all sections are correctly
% sorted in the column col_number).
tini = tic;
numstars = 0;
for k=1:params.numsections
    
    [starcat, filename] = ...
        load_CatalogSection_Raw(params, k);
    
    % Checks for NaNs (IEEE-754 not a number) in the star catalog
    % NaNs = indeterminates = 0/0, +inf - inf, etc.
    if sum(sum(isnan(starcat))) ~= 0
        error('Catalog section %d (in file %s) has one or more NaNs.', ...
            k, filename);
    end
    
    section_size = size(starcat, 1);
    
    % Checks if catalog section column col_number is sorted
    if issorted(starcat(:, col_number)) == false
        error(['Catalog section %d (in file %s), ' ...
            'column %d is not sorted!'], ...
            k, filename, col_number);
    end
    
    % First element of catalog section k+1 must be strictly larger than
    % last element of section k (by the way these sections were generated
    % in 2mass/sort_2mass_JHK.m and 2mass/sort_2mass_HKJ.m.
    if k > 1
        if ~(last_value < starcat(1, col_number))
            error(['Catalog section %d (in file %s) should start '...
                'with values in column %d strictly larger than the '...
                'last value in the same column in section %d.'], ...
                k, filename, col_number, k - 1);
        end
    end
    numstars = numstars + section_size;
    last_value = starcat(end, col_number);
    clear starcat;
    
    fprintf(' Section %d/%d from file %s is OK.\n', ...
        k, params.numsections, filename);
end

if numstars ~= params.catsize
    error('Found %d stars. Should have %d stars.',...
        numstars, params.catsize);
end

fprintf('Catalog verification performed in %f seconds.\n', toc(tini));

end


function [catOption, ctOpt] = load_CatSect_getOption(cat_option)
% Helper function for load_CatalogSection and load_CatSect_getnames

% Catalog options:
catOption.JH = 1; % these values
catOption.JK = 2; %  must be
catOption.HK = 3; % different.

% Identify catalog option
if strcmp(cat_option, 'JH') == true
    ctOpt = catOption.JH;
elseif strcmp(cat_option, 'JK') == true
    ctOpt = catOption.JK;
elseif strcmp(cat_option, 'HK') == true
    ctOpt = catOption.HK;
else
    error('Unrecognised catalog option ''%s''', cat_option);
end

end


function [catFile, catname] = load_CatSect_getnames(params, numsect)
% Helper function for load_CatalogSection and load_CatalogSection_Raw

[catOption, ctOpt] = load_CatSect_getOption(params.cat_option);

if (ctOpt == catOption.JH) || (ctOpt == catOption.JK)
    % these two catalog options share the same files
    catFile = sprintf('%s%.2d.mat', params.TwoMASSJHK_basename, numsect);
    catname = sprintf('%s%.2d', params.cat_basename_JHK, numsect);
    testIfFileExists(catFile, '2MASS', 'TwoMASSJHK_basename');
elseif ctOpt == catOption.HK
    catFile = sprintf('%s%.2d.mat', params.TwoMASSHKJ_basename, numsect);
    catname = sprintf('%s%.2d', params.cat_basename_HKJ, numsect);
    testIfFileExists(catFile, '2MASS', 'TwoMASSHKJ_basename');
else
    error('Unrecognized catalog option ''%s''(%d)', ...
        params.cat_option, ctOpt);
end

end


function [psc_section, catFile] = load_CatalogSection_Raw(params, numsect)
% loads a star catalog section, without doing any processing.
% input parameters:
%  params = program parameters
%  numsect = number of catalog section to be loaded
% Returns:
% psc_section: an N x 5 matrix with:
%  first  column => right ascension in degrees
%  second column => declination in degrees
%  third  column => magnitude in band J
%  fourth column => magnitude in band H
%  fifth  column => magnitude in band Ks
% catFile => Filename of the catalog section.

% Gets section file name and name of the matrix holding the catalog section
% inside that file
[catFile, catname] = load_CatSect_getnames(params, numsect);

% loads PSC_JHK?? from PSC_JHK??.mat or
%       PSC_HKJ?? from PSC_HKJ??.mat
load(catFile);
  % psc_section(:,1) == star R.A in degrees
  % psc_section(:,2) == star dec. in degrees
  % psc_section(:,3) == magnitude in band J
  % psc_section(:,4) == magnitude in band H
  % psc_section(:,5) == magnitude in band K

psc_section = eval([catname ';']);

end
    


function [starcat, catFile] = load_CatalogSection(params, ... 
    numsect, sort_order, showCatMsg)
% loads and pre-process a star catalog section.
% input parameters:
%  params = program parameters
%  numsect = number of catalog section to be loaded
%  sort_order = an up to 5 x 1 vector with sorting order. To do no sorting
%               of the catalog section, pass an empty array in sort_order.
%               This is an example of an empty array:   no_sort = [];
%  showCatMsg = if true, detailed messages are printed.
% Returns:
% starcat: an N x 4 matrix with:
%  first  column => right ascension in radians
%  second column => declination in radians
%  third  column => magnitude in band b1
%  fourth column => magnitude in band b2
% catFile => Filename of the catalog section.

% Catalog options:
[catOption, ctOpt] = load_CatSect_getOption(params.cat_option);

if showCatMsg
    if ctOpt == catOption.JH
        display (', using J and H magnitudes.');
    elseif ctOpt == catOption.JK
        display (', using J and Ks magnitudes.');
    elseif ctOpt == catOption.HK
        display (', using H and Ks magnitudes.');
    else
        error('Unrecognised catalog option ''%s''(%d)',...
            params.cat_option, ctOpt);
    end
end

% Gets section file name and name of the matrix holding the catalog section
% inside that file
[catFile, catname] = load_CatSect_getnames(params, numsect);

% loads PSC_JHK?? from PSC_JHK??.mat or
%       PSC_HKJ?? from PSC_HKJ??.mat
if showCatMsg
    fprintf(' Loading section from file %s\n', catFile);
end
load(catFile);
  % PSC_XXXYY(:,1) == star R.A. in degrees
  % PSC_XXXYY(:,2) == star dec. in degrees
  % PSC_XXXYY(:,3) == magnitude in band J
  % PSC_XXXYY(:,4) == magnitude in band H
  % PSC_XXXYY(:,5) == magnitude in band Ks
 % being XXX = JHK or HJK and YY == 01 to 20.
% Sorting star catalog section (if sort_order is not an empty matrix)
if (size(sort_order, 1)) == 0
    if showCatMsg
        fprintf(' Keeping section original order.\n');
    end
    psc_section = eval([catname ';']);
else
    if showCatMsg
        fprintf(' Sorting section by [');
        fprintf('%d ', sort_order);
        fprintf('] order.\n');
    end
    psc_section = eval(['sortrows(' catname ', sort_order);']);
end
eval(['clear ' catname ';']);

% Selects bands to be used in accordance to catOpt:
if ctOpt == catOption.JH
    starcat = [deg2rad(psc_section(:,1)), deg2rad(psc_section(:,2)),...
               psc_section(:,3),  psc_section(:,4)];
    % starcat(:,1) == star R.A. in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in J band (band 1)
    % starcat(:,4) == star magnitude in H band (band 2)
    
elseif ctOpt == catOption.JK
    starcat = [deg2rad(psc_section(:,1)), deg2rad(psc_section(:,2)),...
               psc_section(:,3),  psc_section(:,5)];
    % starcat(:,1) == star R.A. in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in J band (band 1)
    % starcat(:,4) == star magnitude in Ks band (band 2)
    
elseif ctOpt == catOption.HK
    starcat = [deg2rad(psc_section(:,1)), deg2rad(psc_section(:,2)),...
               psc_section(:,4),  psc_section(:,5)];
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in H band (band 1)
    % starcat(:,4) == star magnitude in Ks band (band 2)
end

end


function testIfFileExists(pathToFile, catName, fieldName)
% Helper function for load_Catalog

fileExistFlag = ...
    exist(pathToFile, 'file') || ...
    exist([pathToFile '.mat'], 'file');

if ~fileExistFlag
    error (['Catalog section "%s" containing data from %s not found!\n'...
        'Please specify the correct base name and location of this'...
        ' and other sections by setting correctly the fields in'...
        ' the second input argument passed to this function'...
        ' (field input_params.%s).'],...
        pathToFile, catName, fieldName);
end
end



function [cat_Ti, cat_Ci, nclamp] = calc_equiv_blackbodies(...
    starcat, BB, b2name, verbose)
% Computes the equivalent black-body models for each star in the star
% catalog.
% inputs:
%   starcat = star catalog
%   BB = struct containing curve relating color indices with temperature
%   b2name = name of the secondary band.
%   verbose = verbosity flag => If true, detailed text is printed.
% outputs:
%   Ti = temperatures of the black-body equivalents for each star.
%   Ci = dilution factor for the black-body equivalents.
%   nclamp = number of stars that had their magnitudes in band 2 adjusted
%            so that their color index were within the range specified by
%            the color index of the hottest and coldest black-body in BB.
%
% Limitation: The curve relating the color indices (BB.color) with
% reciprocal of temperature (BB.Tinv) must be strictly increasing 
% monotonic (higher temperature meaning lower color indexes).

% Lets first check if the curve relating the color indices with reciprocal
% of temperature is strictly increasing. If this is the case, then if
% BB.Tinv is strictly increasing, BB.color should also be strictly
% increasing.

% By construction BB.Tinv should be strictly increasing, and should not
% contain NaN's (IEEE Not a Number)
assert(sum(isnan(BB.Tinv))==0);
Tinv_diff = diff(BB.Tinv);
assert(min(Tinv_diff) > 0);

% Lets check if BB.color is also strictly increasing
assert(sum(isnan(BB.color)) == 0);
color_diff = diff(BB.color);
if ~(min(color_diff) > 0)
    % Notice about error message below: increasing Tinv (temperature
    % reciprocal) means decreasing temperature, so decreasing color index
    % with temperature means increasing color index with Tinv and
    % BB.color in increasing order.
    error(['The computed %s color index is not monotonically' ...
        ' decreasing with temperature in the range %f K < T < %f K.' ...
        ' This algorithm does not work in such situations.' ...
        ' Perhaps the variable "band_order" was set incorrectly in' ...
        ' the internal function loadCatBands.'], ...
        BB.colorIdxName, BB.T(end), BB.T(1));
    % If BB.color is strictly decreasing with reciprocal of temperature,
    % the problem can be easily solved by changing the sign of band_order
    % and the band order in function loadCatBands.
    % If this is not the case (BB.color not strictly monotonic), then
    % there's no easy solution.
end

% Computes the color index for all stars in the catalog:
if BB.band_order > 0
    colorIdx = starcat(:,3) - starcat(:,4); % band1 - band2
else
    colorIdx = starcat(:,4) - starcat(:,3); % band2 - band1
end

% Counts the number of stars that will have their band2 magnitudes adjusted
bluer_star_count  = sum(colorIdx < BB.color(1));
redder_star_count = sum(colorIdx > BB.color(end));
nclamp = bluer_star_count + redder_star_count;

if verbose
    fprintf( ['%d stars with %s < %f (Teq > %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        bluer_star_count, BB.colorIdxName, BB.color(1), BB.T(1), ...
        BB.colorIdxName, BB.color(1), b2name);
    fprintf( ['%d stars with %s > %f (Teq < %g K) had their' ...
        ' %s color index\n'...
        ' set to %f and their %s magnitudes adjusted accordingly.\n'], ...
        redder_star_count, BB.colorIdxName, BB.color(end), BB.T(end), ...
        BB.colorIdxName, BB.color(end), b2name);
end

% Clamps color indexes.
colorIdxClamped = min(max(colorIdx, BB.color(1)),BB.color(end));

% From that, compute the equivalent black-body temperatures for all 
% stars, by interpolating over the relation between color index and
% inverse of temperature for black-bodies
cat_Tinv = interp1(BB.color, BB.Tinv, colorIdxClamped, 'pchip', 0);
cat_Ti = 1./ cat_Tinv;

% Computing the apparent magnitude in band1 that the equivalent 
% black-body that represents each star would have for an observer located
% on its surface. cat_BB_b1mag will be typically between -30 and -45
cat_BB_b1mag = interp1(BB.Tinv, BB.b1mag, cat_Tinv, 'pchip', 0); 

% Compute dilution factor for all stars:
cat_Ci = 10 .^ (0.4 * (cat_BB_b1mag - starcat(:,3)));

end



function Nlist = adjust_Nlist(Nlist_in, catsize)
% Creates a copy of Nlist_in truncated to catsize, with last entry equal to
% catsize

n = size(Nlist_in, 2);
Nlist = zeros(1, n+1);

k = 1;
while (k <= n) && (Nlist_in(k) < catsize)
    Nlist(k) = Nlist_in(k);
    k = k+1;
end

Nlist(k) = catsize;
Nlist = Nlist(1:k);

end


%%%%%%%%%%%%%% Functions for plotting figures %%%%%%%%%%%%%%%%%%%%%%%%%%


function plotBands(BB, b1name, b2name, plotStyle)

figure;
plot(BB.lambda, BB.B1, [plotStyle.b1Color '-']);
hold on;
plot(BB.lambda, BB.B2, [plotStyle.b2Color '--']);
grid on;
grid minor;
set (gca,'MinorGridAlpha', get(gca,'GridAlpha')*0.25);
set (gca,'MinorGridLine', get(gca,'GridLine'));
xlabel('wavelength / m');
ylabel('normalized energy response');
title('Passbands spectral response');
set(gcf,'Units','inches');
figsize=[4 3]; % figure width and height in inches.
set(gcf,'Position', [2, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
legend(b1name, b2name, 'location', 'best');

%imgfname = [b1name b2name 'response'];
%print(gcf, imgfname, '-depsc2');  % save figure to file
%print(gcf, imgfname, '-dmeta');  % save figure to file - Windows specific.
%print(gcf, imgfname, '-dpdf');  % save figure to file

end



function plotFmat(FsumList, Nlist, catname, b1name, b2name)

Nlistsize = size(Nlist, 2);
Ftrace = zeros(1, Nlistsize);

for k=1:Nlistsize
    Ftrace(k) = trace(FsumList(:,:,k));
end

figure;
plot (Nlist, Ftrace, 'ko-');
set(gca, 'xscale', 'log');
grid on;
title(['For ' catname ' with stars sorted by ' ...
    b1name ' and ' b2name ' mag.']);
xlabel('Number of stars');
ylabel('trace of the Fisher Information matrix');

end



function plotInvCovMat(FsumList, Nlist, catname, b1name, b2name)

Nlistsize = size(Nlist, 2);
Covtrace = zeros(1, Nlistsize);

for k=1:Nlistsize
    Covtrace(k) = trace(inv(FsumList(:,:,k)));
end

figure;
plot (Nlist, 1 ./ Covtrace, 'ko-');
set(gca, 'xscale', 'log');
grid on;
title(['For ' catname ' with stars sorted by ' ...
    b1name ' and ' b2name ' mag.']);
xlabel('Number of stars');
ylabel('1 / trace( covariance matrix )');

end