function outputs = lower_bound_from_starcat(catOpt, input_params, verbose)
% LOWER_BOUND_FROM_STARCAT
% Computes an estimate of the lower bound on star tracker accuracy, using
% data from star catalog 'catalog' and the black-body model for stars
% described in the STR Limits paper.
%
% Syntax:
%   outputs = LOWER_BOUND_FROM_STARCAT(catOpt)
%   outputs = LOWER_BOUND_FROM_STARCAT(catOpt, input_params)
%   outputs = LOWER_BOUND_FROM_STARCAT(catOpt, input_params, verbose)
%
% Inputs:
%  catOpt: It can be one of the following:
%    'HipBV' = Using B-V magnitudes from the Hipparcos star catalog.
%    'HipVI' = Using V-I magnitudes from the Hipparcos star catalog.
%    'Tycho2' = Using BT and VT magnitudes from the Tycho-2 star catalog.
%    '2MASS100k_JH' = Using J and H magnitudes from the the brightest
%            100,000 stars in J magnitude from the 2-MASS star catalog.
%    '2MASS100k_JK' = Using J and Ks magnitudes from the the brightest
%            100,000 stars in J magnitude from the 2-MASS star catalog.
%    '2MASS100k_HK' = Using H and Ks magnitudes from the the brightest
%            100,000 stars in J magnitude from the 2-MASS star catalog.
%
%  input_params: A struct with configuration data for this function.
%            If it is not specified, or if there are missing fields,
%            defaults are used.
%   Description of fields in input_params:
%    diam = ideal spherical star tracker diameter in meters. (default = 1)
%    texposure = exposure time in seconds. (default = 1)
%    Tmin = minimum temperature in kelvins for equivalent black-bodies
%           that represent a star from the catalog. (default = 2000)
%    Tmax = maximum temperature in kelvins for equivalent black-bodies
%           that represent a star from the catalog. (default = 30000)
%
%  verbose: if true, detailed diagnostic messages are printed during
%           execution. (default = false)
%
% Output:
%  outputs: a structure containing data generated by this function.
%
% Example:
%  To process Tycho-2 star catalog with star tracker diameter of 0.1 m and
%  exposure time of 100 ms, enter the following commands:
%   >> param.diam = 0.1;
%   >> param.texposure = 0.1;
%   >> outputs = STR_Lower_bound_from_cat('Tycho2', param);
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.


% References:
%
% Ref1 - F. L. Markley, J. L. Crassidis. Fundamentals of Spacecraft
%        Attitude Determination and Control. Springer. New York, 2014.
%        ISBN 978-1-4939-0801-1.


t1 = tic;

% Default configurations
% Can be overriden by specifying equivalent field names in input_params.
default_params = load_default_params();

% Processing input parameters.
if nargin < 2
    params = default_params;
else
    params = merge_input_params(input_params, default_params);
end

if nargin < 3
    verbose = false;
end

params.cat_option = catOpt; % saves some input arguments as parameters
params.verbose = verbose; % saves some input arguments as parameters

if verbose
    print_params(params);
end


% Copy some internal variables to the output, for debugging purposes.
if nargin >= 2
    outputs.input_params = input_params;
end

outputs.default_params = default_params;
outputs.params = params;
clear input_params;
clear default_params;

% Checks program parameters for consistency
check_params(params);

% loads Physical and Mathematical constants
consts = loadConsts(verbose);

% Select spectral bands to use, depending on the selected catalog.
[b1, b2, band_order, plotStyle] = loadCatBands(catOpt, verbose);

% Compute reference fluxes for bands b1 and b2
b1.Ref = VegaRefmag(b1.data, b1.Vegamag);
b2.Ref = VegaRefmag(b2.data, b2.Vegamag);

if verbose
    fprintf('Reference flux for band %s = %e W/m^2\n', b1.name, b1.Ref);
    fprintf('Reference flux for band %s = %e W/m^2\n', b2.name, b2.Ref);
end

% Compute black-body magnitudes. Also returns in struct BB temperatures
% and wavelength tables for interpolation and numerical integration.
% BB is an acronym for black-body.
BB = calc_BlackBody_mags(params, consts, b1, b2, band_order, verbose);

if params.plotBands
    % Plot bands spectral responses
    plotBands(BB, b1.name, b2.name, plotStyle);
end

% Loads and process star catalog
fprintf('Loading star catalog :');
[starcat, starcatname] = load_Catalog(params, true);

if params.sortByMagnitude
    % Sorts star catalog first by band1 magnitudes, and if these are equal
    % then by band2 magnitudes.
    fprintf(['Sorting star catalog by %s magnitudes and,' ...
        ' if equal, by %s magnitudes.\n'], b1.name, b2.name);
    starcat = sortrows(starcat, [3 4]);
end

disp('Computing equiv. black-bodies for all stars in the catalog.');
[Ti, Ci, nclampTop, nclampBot] = calc_equiv_black_bodies(...
    starcat(:,3), starcat(:,4), BB, b2.name, verbose);
nclamp = nclampTop + nclampBot;
outputs.nclampTop = nclampTop;
outputs.nclampBot = nclampBot;

catsize = size(starcat, 1); % catalog length
Nlist = adjust_Nlist(params.Nlist, catsize);

% Computes the number of photons for each star, if requested by the user
if params.calcNumPhotons
    area_times_texp = pi/4 * params.diam^2 * params.texposure;
    outputs.numPhotons = consts.tauSB * area_times_texp * Ci .* (Ti .^ 3);
    toc(t1);
end

% Computes the lower bound on centroiding error for each star
% cat_F is the reciprocal of the variance lower bound = 1/sigma_min^2
disp('Computing lower bound on centroiding accuracy for all stars.');
InfoFactor = consts.InfoFactor * params.diam^4 * params.texposure;
outputs.InfoFactor = InfoFactor;
cat_F = InfoFactor  .* Ci .* (Ti .^ 5);

% Computes unit vectors for every star in the star catalog
% Notice: angles in starcat must be in radians!
disp('Computing unit vector representation for all stars.');
catvect = zeros(catsize, 3);
catvect(:,1) = cos(starcat(:, 1)) .* cos(starcat(:, 2)); % X component
catvect(:,2) = sin(starcat(:, 1)) .* cos(starcat(:, 2)); % Y component
catvect(:,3) = sin(starcat(:, 2)); % Z component of unit vector

% Computes the Fisher information matrix F (equation 5.114 in Ref. 1)
% To reduce numerical precision loss from summing small values into a
% large sum, the summation is partitioned into bins specified in Nlist.
% To speed up computations, the bins are partitioned in sections having
% up to params.maxChunckSize elements and the Fisher information
% contributed by that segment is computed in vectorized form using function
% calcFisherMat.
disp('Computing Fisher information matrix.');
assert(Nlist(end) == catsize);
Nlistsize = size(Nlist, 2);
F = zeros(3); % cumulative Fisher information matrix
Fbin = zeros(3); % Fisher information matrix for stars in a bin.
FbinList = zeros(3, 3, Nlistsize);
FsumList = zeros(3, 3, Nlistsize);
Nlistpos = 1;
s = 1;  % star index and loop counter.
nextNlistVal = Nlist(Nlistpos);
while (s <= catsize)
    chunkSize = min(params.maxChunkSize, nextNlistVal + 1 - s);
    assert(chunkSize > 0);
    s_end = s + chunkSize - 1;
    Fbin = Fbin + calcFisherMat(catvect(s:s_end, :)', cat_F(s:s_end)');
    if s_end == nextNlistVal
        F = F + Fbin;
        FbinList(:,:,Nlistpos) = Fbin;
        FsumList(:,:,Nlistpos) = F;
        if s_end < catsize
            Fbin = zeros(3);
            Nlistpos = Nlistpos+1;
            nextNlistVal = Nlist(Nlistpos);
        end
    end
    s = s_end + 1;
end
assert(s_end == catsize);

% Computes the covariance matrix - equation 5.113 in Ref. 1
disp('Computing lower bound on attitude accuracy:');
Pthetatheta = inv(F);

% Computes the variance on the attitude error
var_theta = trace(Pthetatheta);

% Computes and prints the attitude error standard deviation
sigma_theta = sqrt(var_theta);

fprintf('sigma_attitude = %.16g rad = %.16g mas\n', sigma_theta,...
    sigma_theta * 180/pi * 3600 * 1000);

% Plotting some results.
if params.plotFmat
    plotFmat(FsumList, Nlist, starcatname, b1.name, b2.name,...
        params.sortByMagnitude);
end

if params.plotCovmat
    plotInvCovMat(FsumList, Nlist, starcatname, b1.name, b2.name,...
        params.sortByMagnitude);
end

% Pass some internal variables to the output.
if nargout > 0
    disp('Copying some internal variables to the output');
end
outputs.consts = consts;
outputs.b1 = b1;
outputs.b2 = b2;
outputs.plotStyle = plotStyle;
outputs.BB = BB;
if params.saveCat
    outputs.starcat = starcat;
    outputs.Ti = Ti;
    outputs.Ci = Ci;
    outputs.cat_F = cat_F;
end
outputs.nclamp = nclamp;
outputs.Nlist = Nlist;
outputs.FbinList = FbinList;
outputs.FsumList = FsumList;
outputs.F = F;

toc(t1);

end



%%%%%%%%%%%%%%%%%%%%%% Functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function default_params = load_default_params
% Default configurations
default_params.diam = 1;  % STR diameter in meters
default_params.texposure = 1; % STR exposure time in seconds
default_params.Tmin = 2000;  % Minimum black-body temperature in kelvins.
default_params.Tmax = 30000; % Maximum black-body temperature in kelvins.
default_params.TinvStep = 4e-7; % inverse of temperature step when 
  % generating a list of black bodies with temperatures from Tmin to Tmax.
default_params.log10integStep = 0.0001; % base-10 logarithm of integration
  % step in wavelength.
default_params.sortByMagnitude = true; % If true, stars are sorted by
  % magnitude in band1 prior to computations.
default_params.Nlist = [2, 10, 100, 1000, 10000, 100000, 1000000,...
    10000000, 100000000, 1000000000, 10000000000];
default_params.maxChunkSize = 10000; % Maximum size of catalog sections
  % sent to calcFisherMat.
default_params.HipBVI_matfile = 'outputs/hipBVI.mat'; % Path to processed
  % Hipparcos catalog .mat file with V magnitudes, B-V color indexes and
  % V-I color indexes, generated by calc_HipBVI.m and read_Hipparcos_bvi.m.
default_params.Tycho2_matfile = 'outputs/tycho2.mat'; % Path to processed
  % Tycho-2 catalog .mat file with BT and VT magnitudes. Generated by
  % read_Tycho2.m.
default_params.TwoMASS100k_matfile = '2mass_100k.mat';
default_params.TwoMASSJHK01_matfile = 'c:\2mass\data\psc_JHK01.mat';
default_params.saveCat = false;
default_params.plotBands = false;
default_params.plotFmat = false;
default_params.plotCovmat = false;
default_params.calcNumPhotons = false;
end



function check_params(params)
% Checks program parameters for consistency

% Checking if params.Nlist is monotonically increasing
Nlistdiff = diff(params.Nlist);
if min(Nlistdiff) < 1
    error('params.Nlist must be a strictly increasing vector of integers');
end

end



function print_params(params)

disp(['Running function STR_Lower_bound_from_cat with the following '...
    'parameters:']);

fnames = fieldnames(params);
formatString = sprintf(' params.%%-%ds = ', getLongestStringLen(fnames));
N = size(fnames, 1);
for k=1:N
    fieldname = fnames{k};
    fprintf(formatString, fieldname);
    disp(params.(fieldname));
end
end



function [maxlen, N] = getLongestStringLen(textlist)
% [maxlen, N] = getLongestStringLen(textlist)
% Returns the length of the longest string in a list
% maxlen is the length of the longest string.
% N is the number of strings in that list.

if size(textlist, 2) ~= 1  ||  ~ismatrix(textlist)
    error('Cannot handle multidimensional text lists');
end
N = size(textlist, 1);
maxlen = 0;
for k=1:N
    if size(textlist{k}, 1) ~= 1  ||  ~ismatrix(textlist{k})
        error('Each element in the text list must be a row vector');
    end
    maxlen = max(maxlen, size(textlist{k}, 2));
end
end



function [b1, b2, band_order, plotStyle] = loadCatBands(catOpt, verbose)
% Select spectral bands according to star catalog option.
%  b1 is the primary spectral band.
%  b2 is the secondary spectral band.
%  band_order == +1 if the first band is at a shorter wavelength than
%                   band 2, and -1 otherwise.
%  plotStyle is the plot style for spectral bands b1 and b2

[bList, Vegamag] = loadBands(); % run function in loadBands.m

if verbose
    fprintf('Loading spectral band for option %s\n', catOpt);
end

if strcmp(catOpt, 'HipBV') == true
    band_order = -1;
    
  % The primary spectral band is the Johnson's V (visual) passband.
    b1.name = 'V';
    b1.data = bList.V;
    b1.Vegamag = Vegamag.V;
    plotStyle.b1Color = 'k'; % plot this band in black
    
  % The secondary spectral band is the Johnson's B (blue) passband.
    b2.name = 'B';
    b2.data = bList.B;
    b2.Vegamag = Vegamag.B;
    plotStyle.b2Color = 'b'; % plot this band in blue
    
elseif strcmp(catOpt, 'HipVI') == true
    band_order = +1;
    
  % The primary spectral band is the Johnson's V (visual) passband.
    b1.name = 'V';
    b1.data = bList.V;
    b1.Vegamag = Vegamag.V;
    plotStyle.b1Color = 'k'; % plot this band in black
    
  % The secondary spectral band is the Johnson's I (infrared) passband.
    b2.name = 'I';
    b2.data = bList.I;
    b2.Vegamag = Vegamag.I;
    plotStyle.b2Color = 'r'; % plot this band in red
    
elseif strcmp(catOpt, 'Tycho2') == true
    band_order = -1;
    
  % Taking the Tycho-2 VT (visual) passband as the primary, since it was
  % less subject to degradation due to ionizing radiation in the Hipparcos
  % spacecraft.
    b1.name = 'V_T';
    b1.data = bList.VT;
    b1.Vegamag = Vegamag.VT;
    plotStyle.b1Color = 'k'; % plot this band in black
    
  % The secondary spectral band is the Tycho-2 BT passband.
    b2.name = 'B_T';
    b2.data = bList.BT;
    b2.Vegamag = Vegamag.BT;
    plotStyle.b2Color = 'b'; % plot this band in blue

elseif (strcmp(catOpt, '2MASS100k_JH') == true)  || ...
       (strcmp(catOpt, '2MASS_JH01'  ) == true)
    band_order = 1;
    
    b1.name = 'J';
    b1.data = bList.J;
    b1.Vegamag = Vegamag.J;
    plotStyle.b1Color = 'b'; % plot this infrared band in blue
    
    b2.name = 'H';
    b2.data = bList.H;
    b2.Vegamag = Vegamag.H;
    plotStyle.b2Color = 'k'; % plot this band in black

elseif (strcmp(catOpt, '2MASS100k_JK') == true) || ...
       (strcmp(catOpt, '2MASS_JK01'  ) == true)
    band_order = 1;
    
    b1.name = 'J';
    b1.data = bList.J;
    b1.Vegamag = Vegamag.J;
    plotStyle.b1Color = 'b'; % plot this infrared band in blue
    
    b2.name = 'K';
    b2.data = bList.K;
    b2.Vegamag = Vegamag.K;
    plotStyle.b2Color = 'r'; % plot this band in red

elseif (strcmp(catOpt, '2MASS100k_HK') == true) || ...
       (strcmp(catOpt, '2MASS_HK01'  ) == true)
    band_order = 1;
    
    b1.name = 'H';
    b1.data = bList.H;
    b1.Vegamag = Vegamag.H;
    plotStyle.b1Color = 'k'; % plot this infrared band in black
    
    b2.name = 'K';
    b2.data = bList.K;
    b2.Vegamag = Vegamag.K;
    plotStyle.b2Color = 'r'; % plot this band in red
    
else
    error('Unrecognised catalog option ''%s''', catOpt);
end

if verbose
    fprintf(' Primary passband:\n  ');
    disp(b1);
    fprintf(' Secondary passband:\n  ');
    disp(b2);
end

end



function [starcat, starcatname] = load_Catalog(params, showCatMsg)
% loads and pre-process a star catalog.
% Returns starcat: an N x 4 matrix with:
%  first  column => right ascension in radians
%  second column => declination in radians
%  third  column => magnitude in band b1
%  fourth column => magnitude in band b2

catOpt = params.cat_option;
starcatname = catOpt; % default catalog name, if not specified later.

if strcmp(catOpt, 'HipBV') == true
    if showCatMsg
        display (' Hipparcos with B-V color index.');
    end
    catFile = params.HipBVI_matfile;
    testIfFileExists(catFile, 'Hipparcos', 'B-V', 'HipBVI_matfile');
    
    load(catFile); % created with calc_HipBVI.m
    % HipBVI(:,1) == Hip. catalog number
    % HipBVI(:,2) == star R.A in degrees
    % HipBVI(:,3) == star dec. in degrees
    % HipBVI(:,4) == star magnitude in V band
    % HipBVI(:,5) == star B-V color index
        
    starcat = [deg2rad(HipBVI(:,2)), deg2rad(HipBVI(:,3)),...
               HipBVI(:,4) ,  HipBVI(:,4) + HipBVI(:,5)]...
               ; %#ok<NODEF> - HipBVI loaded from .mat file.
           
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in V band (band 1)
    % starcat(:,4) == star magnitude in B band (band 2)
    
elseif strcmp(catOpt, 'HipVI') == true
    if showCatMsg
        display (' Hipparcos using V-I color index.');
    end
    catFile = params.HipBVI_matfile;
    testIfFileExists(catFile, 'Hipparcos', 'V-I', 'HipBVI_matfile');
    
    load(catFile); % created with calc_HipBVI.m
    % HipBVI(:,1) == Hip. catalog number
    % HipBVI(:,2) == star R.A in degrees
    % HipBVI(:,3) == star dec. in degrees
    % HipBVI(:,4) == star magnitude in V band
    % HipBVI(:,6) == star V-I color index
    
    starcat = [deg2rad(HipBVI(:,2)), deg2rad(HipBVI(:,3)),...
               HipBVI(:,4) ,  HipBVI(:,4) - HipBVI(:,6)]...
               ; %#ok<NODEF> - HipBVI loaded from .mat file.
    
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in V band (band 1)
    % starcat(:,4) == star magnitude in I band (band 2)
            
elseif strcmp(catOpt, 'Tycho2') == true
    if showCatMsg
        display (' Tycho-2 using BT and VT magnitudes.');
    end
    catFile = params.Tycho2_matfile;
    testIfFileExists(catFile, 'Tycho-2', 'BT-VT', 'Tycho2_matfile');
    
    load(catFile); % created with calc_Tycho2.m
    % starcat(:,1) == position of star in the Tycho-2 catalog (row number)
    % starcat(:,2) == star R.A in degrees
    % starcat(:,3) == star dec. in degrees
    % starcat(:,4) == Tycho BT magnitude
    % starcat(:,5) == Tycho VT magnitude
    
    starcat = [deg2rad(Tyc2(:,2)), deg2rad(Tyc2(:,3)),...
               Tyc2(:,5),  Tyc2(:,4)]...
               ; %#ok<NODEF> - Tyc2 loaded from .mat file.
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in VT band (band 1)
    % starcat(:,4) == star magnitude in BT band (band 2)
    
elseif strcmp(catOpt, '2MASS100k_JH') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using J and H magnitudes.');
    end
    catFile = params.TwoMASS100k_matfile;
    testIfFileExists(catFile, '2MASS', 'J-H', 'TwoMASS100k_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m
    % starcat(:,1) == star R.A in degrees
    % starcat(:,2) == star dec. in degrees
    % starcat(:,3) == 2MASS J magnitude
    % starcat(:,4) == 2MASS H magnitude
    % starcat(:,5) == 2MASS Ks magnitude
    
    starcat = [deg2rad(psc_test(:,1)), deg2rad(psc_test(:,2)),...
               psc_test(:,3),  psc_test(:,4)]; %#ok<NODEF> - psc_test
                                               % loaded from catfile.
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in J band (band 1)
    % starcat(:,4) == star magnitude in H band (band 2)
    
elseif strcmp(catOpt, '2MASS100k_JK') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using J and Ks magnitudes.');
    end
    catFile = params.TwoMASS100k_matfile;
    testIfFileExists(catFile, '2MASS', 'J-K', 'TwoMASS100k_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m
    % starcat(:,1) == star R.A in degrees
    % starcat(:,2) == star dec. in degrees
    % starcat(:,3) == 2MASS J magnitude
    % starcat(:,4) == 2MASS H magnitude
    % starcat(:,5) == 2MASS Ks magnitude
    
    starcat = [deg2rad(psc_test(:,1)), deg2rad(psc_test(:,2)),...
               psc_test(:,3),  psc_test(:,5)]; %#ok<NODEF> - psc_test
                                               % loaded from catfile.
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in J band (band 1)
    % starcat(:,4) == star magnitude in Ks band (band 2)
    
elseif strcmp(catOpt, '2MASS100k_HK') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using H and Ks magnitudes.');
    end
    catFile = params.TwoMASS100k_matfile;
    testIfFileExists(catFile, '2MASS', 'H-K', 'TwoMASS100k_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m
    % starcat(:,1) == star R.A in degrees
    % starcat(:,2) == star dec. in degrees
    % starcat(:,3) == 2MASS J magnitude
    % starcat(:,4) == 2MASS H magnitude
    % starcat(:,5) == 2MASS Ks magnitude
    
    starcat = [deg2rad(psc_test(:,1)), deg2rad(psc_test(:,2)),...
               psc_test(:,4),  psc_test(:,5)]; %#ok<NODEF> - psc_test
                                               % loaded from catfile.
    % starcat(:,1) == star R.A in radians
    % starcat(:,2) == star dec. in radians
    % starcat(:,3) == star magnitude in H band (band 1)
    % starcat(:,4) == star magnitude in Ks band (band 2)

% options '2MASS_JH01', '2MASS_JH01' and '2MASS_HK01' use the same catalog
% format of the previous options, with the exception that the star catalog
% is larger.
elseif strcmp(catOpt, '2MASS_JH01') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using J and H magnitudes.');
    end
    catFile = params.TwoMASSJHK01_matfile;
    testIfFileExists(catFile, '2MASS', 'J-H', 'TwoMASSJHK01_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m   
    psc_cat = psc_JHK01;
    clear psc_JHK01;
    starcat = [deg2rad(psc_cat(:,1)), deg2rad(psc_cat(:,2)),...
               psc_cat(:,3),  psc_cat(:,4)]; 

elseif strcmp(catOpt, '2MASS_JK01') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using J and Ks magnitudes.');
    end
    catFile = params.TwoMASSJHK01_matfile;
    testIfFileExists(catFile, '2MASS', 'J-K', 'TwoMASSJHK01_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m   
    psc_cat = psc_JHK01;
    clear psc_JHK01;
    starcat = [deg2rad(psc_cat(:,1)), deg2rad(psc_cat(:,2)),...
               psc_cat(:,3),  psc_cat(:,5)]; 

elseif strcmp(catOpt, '2MASS_HK01') == true
    starcatname = '2MASS';
    if showCatMsg
        display (' 2MASS using H and Ks magnitudes.');
    end
    catFile = params.TwoMASSJHK01_matfile;
    testIfFileExists(catFile, '2MASS', 'H-K', 'TwoMASSJHK01_matfile');
    
    load(catFile); % created with read_2mass_mat.c and convert_all_m.m   
    psc_cat = psc_JHK01;
    clear psc_JHK01;
    starcat = [deg2rad(psc_cat(:,1)), deg2rad(psc_cat(:,2)),...
               psc_cat(:,4),  psc_cat(:,5)]; 

else
    error('Unrecognised catalog option ''%s''', catOpt);
end

end


function testIfFileExists(pathToFile, catName, colorIdxText, fieldName)
% Helper function for load_Catalog

fileExistFlag = ...
    exist(pathToFile, 'file') || ...
    exist([pathToFile '.mat'], 'file');

if ~fileExistFlag
    error (['Catalog file "%s" containing data from %s with'...
        ' %s color indexes not found!\n'...
        'Please specify the correct name and location of this file'...
        ' by setting correctly the field %s in the second'...
        ' input argument passed to this function'...
        ' (field input_params.%s).'],...
        pathToFile, catName, colorIdxText, fieldName, fieldName);
end
end



function Nlist = adjust_Nlist(Nlist_in, catsize)
% Creates a copy of Nlist_in truncated to catsize, with last entry equal to
% catsize

n = size(Nlist_in, 2);
Nlist = zeros(1, n+1);

k = 1;
while (k <= n) && (Nlist_in(k) < catsize)
    Nlist(k) = Nlist_in(k);
    k = k+1;
end

Nlist(k) = catsize;
Nlist = Nlist(1:k);

end


%%%%%%%%%%%%%% Functions for plotting figures %%%%%%%%%%%%%%%%%%%%%%%%%%


function plotBands(BB, b1name, b2name, plotStyle)

figure;
plot(BB.lambda, BB.B1, [plotStyle.b1Color '-']);
hold on;
plot(BB.lambda, BB.B2, [plotStyle.b2Color '--']);
grid on;
grid minor;
set (gca,'MinorGridAlpha', get(gca,'GridAlpha')*0.25);
set (gca,'MinorGridLine', get(gca,'GridLine'));
xlabel('wavelength / m');
ylabel('normalized energy response');
title('Passbands spectral response');
set(gcf,'Units','inches');
figsize=[4 3]; % figure width and height in inches.
set(gcf,'Position', [2, 1, figsize(1), figsize(2)]);
set(gcf,'PaperSize', figsize);
set(gcf,'PaperUnits','inches','PaperPosition',[0, 0, figsize]);
legend(b1name, b2name, 'location', 'best');

%imgfname = [b1name b2name 'response'];
%print(gcf, imgfname, '-depsc2');  % save figure to file
%print(gcf, imgfname, '-dmeta');  % save figure to file - Windows specific.
%print(gcf, imgfname, '-dpdf');  % save figure to file

end



function plotFmat(FsumList, Nlist, catname, b1name, b2name, sortmagFlag)

Nlistsize = size(Nlist, 2);
Ftrace = zeros(1, Nlistsize);

for k=1:Nlistsize
    Ftrace(k) = trace(FsumList(:,:,k));
end

figure;
plot (Nlist, Ftrace, 'ko-');
set(gca, 'xscale', 'log');
grid on;
if sortmagFlag
    title(['For ' catname ' with stars sorted by ' ...
        b1name ' and ' b2name ' mag.']);
else
    title(['For ' catname '.']);
end
xlabel('Number of stars');
ylabel('trace of the Fisher Information matrix');

end



function plotInvCovMat(FsumList, Nlist, catname, b1name, b2name,...
    sortmagFlag)

Nlistsize = size(Nlist, 2);
Covtrace = zeros(1, Nlistsize);

for k=1:Nlistsize
    Covtrace(k) = trace(inv(FsumList(:,:,k)));
end

figure;
plot (Nlist, 1 ./ Covtrace, 'ko-');
set(gca, 'xscale', 'log');
grid on;
if sortmagFlag
    title(['For ' catname ' with stars sorted by ' ...
        b1name ' and ' b2name ' mag.']);
else
    title(['For ' catname '.']);
end
xlabel('Number of stars');
ylabel('1 / trace( covariance matrix )');

end