% Matlab/Octave script used to plot figure comparing the theoretical
% limits of star trackers with the approximate typical accuracy of
% real star trackers, with selectable units in the vertical axis.
% Tested with Matlab 2015b and GNU Octave 5.1.0.
%
% Copyright (C) 2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

% Release notes:
%
% v0.4 - 2019-11-24:
%   - Included estimates for two star sensors used together.
%   - Included estimates for single head star sensors using their aperture
%     diameter.
%   - Included estimates for diffraction and photon noise limited star
%     sensors (ideal star sensors), but limited in magnitude and in field
%     of view (assumed circular).
%   - Adjusted plot size.
%   - Adjusted legend labels.
%   - Created lines joining different estimates (single head, two heads,
%     aperture diameter) belonging to the same star sensor.
%
% v0.3 - 2019-11-12:
%   - Updated list of star tracker models;
%   - yaw/pitch and roll noises are now separated;
%   - for some models, aperture diameter is also included.
%
% v0.2 - 2019-09-25: No change.
%
% v0.1 - 2019-09-18:
%   - First public release.


clear;
close all;


%%% General definitions

% Some units
deg = pi / 180;  % one degree in radians
arcmin = deg / 60; % one arc minute
arcsec = arcmin / 60; % one arc second

% Plot settings:
%  Minimum and maximum values of the combined parameter D^2 * t^(1/2),
%  defining the horizontal axis.
pini = 5e-5; 
pend = 0.2;

% Parameters controlling the vertical margins
botfactor = 0.5;
topfactor = 3;

% Unit used in the vertical scale of the plot:
%  arcsec for second of arc, arcmin for arc minute or deg for degree
%  1 for radians
angleunit = arcsec;

% Plot width and height in inches
plotwidth = 6;
plotheight = 6;
screen_posx = 2; % plot position on screen
screen_posy = 1; % plot position on screen



%%% Data to be plotted

% Star tracker table:
%
% DISCLAIMER: This is not intended to provide a benchmark between different
% star tracker manufacturers, but just to provide some values to illustrate
% Figure 8 in the journal article "Theoretical Limits of Star Sensor
% Accuracy".
%
%  Sources are listed at the end of this file, in the reference section
%  (copied from the journal article).
% format:
%  1st column = estimated yaw/pitch rms noise in arc-seconds.
%  2nd column = estimated roll rms noise in arc-seconds.
%  3rd column = estimated/assumed integration time in seconds.
%  4th column = optical aperture diameter in meters.
%               (when not available, filled with nan = NaN = Not-a-Number).
%  5th column = STR_diam = approx. diameter of the smallest sphere that
%               encloses the optical head of the star tracker, not
%               including its baffle.
% Each row is a star tracker model.
STR_data = [...
    % JenaOptronik ASTRO 15 Star Sensor:
    %  Source: "Data Sheet Astro 15.pdf" (see reference [Astro15])
    %  Notice: dimensions without baffle are not available in the
    %  datasheet, but can be estimated from the figure in its first page.
    %   star tracker height is about 400 pixels
    %   star tracker base diameter is about 240 pixels
    %   Assuming that the 192 mm diameter in the datasheet refers to the
    %   base diameter, WITHOUT the mounting flange, then we have:
    %   each pixel = 192 mm / 240 = 0.8 mm => height = 320 mm
    %   Diameter of the smallest enclosing sphere for a cylinder 320 mm
    %   high by 192 mm in diameter => 1 mm * sqrt(320^2 + 192^2) = 373 mm.
    %   Considering that the star tracker is not really a cylinder, but
    %   is narrower at its lens, then a value between 320 mm and 373 mm
    %   would be a better estimate. Choosing 340 mm = 0.34 m.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
         2.5    , 10.0 , 0.25 ,   0.05  , 0.34;

    % Terma star trackers T1 and T2:
    %  Source: reference [Terma2019], files
    %   t1_t2_star_tracker_rev2.pdf and
    %   two-pager_space_t1_star_tracker_options_a4.pdf.
    %  Note 1: Exposure time assumed to be equal to the inverse of the
    %          maximum update rate.
    %  Note 2: STR_diam = 0.97 * sqrt( (92 mm)^2 + (68 mm)^2) => row
    %           "Optical head Dimension" in the second page of the
    %           datasheet. Factor of 0.97 used to get a closer value to
    %           the true diameter of smallest enclosing sphere, considering
    %           that the star tracker shape is not exactly a cylinder.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        1.5     ,  9.0 , 0.1  ,   0.026 , 0.11;  % star tracker T1 with 26 mm aperture option.
        3.5     , 21.0 , 0.2  ,   nan   , 0.11;  % star tracker T2
    
    % Auriga star tracker from Sodern:
    %  Source: file "AURIGA_Datasheet.pdf" (see reference [Auriga])
    %  Note 1: Reading attitude rms errors from "FOV error" row, on the
    %          second page of the datasheet. The following row "Space-time
    %          noise on XY/Z @ 1-sigma" has these values: 6 / 40 Arcsec.
    %  Note 2: Assuming exposure time to be the inverse of output rate
    %  Note 3: Estimated size of smallest enclosing sphere from picture on
    %          the first page of the datasheet:
    %            height with baffle = 2300 pixels
    %            height without baffle = 1300 pixels
    %            height with baffle = 94 mm (from datasheet)
    %            height without baffle =
    %                94 mm * 1300 px / 2300 px = 53 mm
    %   approximate diameter of the smallest enclosing sphere:
    %     0.95 * sqrt(56^2 + 66^2 + 53^2) mm = 95 mm
    %    Multiplication by 0.95 is to compensate for the factor that the
    %    STR is not a cuboid of 56 mm x 66 mm x 53 mm, therefore, the
    %    diameter of the smallest enclosing sphere should be slightly less
    %    than the diagonal of this cuboid.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        2.0     , 11.0 , 0.5  ,   nan   , 0.095;
    
    % A-STR and AA-STR from Leonardo Airborne & Space Systems
    %  Source: file
    %     "Copia_di_A_STR_Autonomous_Star_Trackers_LQ_mm07786_.pdf" 
    %    see( reference [Leonardo])
    %  Note 1: Reading attitude errors from row "NEA (random error) @ 0.1
    %          degree/second tracking rate" on page 4 and converting
    %          3-sigma values to 1-sigma values.
    %  Note 2: Assuming exposure time to be the inverse of output rate.
    %  Note 3: Estimates for the diameter of the smallest enclosing spheres
    %          without baffle:
    %   For A-STR:
    %     from the figure to the left on page 2 in the datasheet:
    %       height plus baffle = 195 pixels = 290.5 mm (from datasheet)
    %        scale factor => 1 pixel = 1.5 mm approx.
    %       height without baffle = 120 pixels => 180 mm
    %       base width/length = 195 mm x 175 mm (from datasheet)
    %      smallest sphere => sqrt(195^2 + 175^2 + 180^2) mm = 318 mm.
    %   For AA-STR:
    %     from the figure to the right on page 2 in the datasheet:
    %       height plus baffle = 288 pixels = 348 mm (from datasheet)
    %        scale factor => 1 pixel = 1.2 mm approx.
    %       height without baffle = 110 pixels => 133 mm
    %       base width/length = 164 mm x 156 mm (from datasheet)
    %      smallest sphere => sqrt(164^2 + 156^2 + 110^2) mm = 252 mm.

    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        2.0     ,  21.0 , 0.1  ,   nan  , 0.318; % A-STR
        2.0     ,  16.5 , 0.1  ,   nan  , 0.252; % AA-STR
        
    % VST-41M from Vectronic Aerospace GmbH
    %  Source: https://vectronic-aerospace.com/star-sensor-vst-41m
    %  (see reference [VST41M]
    %  Note 1: Reading attitude errors from row "Accuracy 2sigma(x,y /z
    %          axis)" and converting 2-sigma values to 1-sigma values.
    %          Actual rms noise may be better, since quoted values might
    %          also include bias.
    %  Note 2: Assuming exposure time to be the inverse of output rate.
    %  Note 3: Estimates for the diameter of the smallest enclosing sphere
    %          without baffle:
    %     from the figure in this webpage
    %       height plus baffle = 440 pixels = 180 mm
    %        scale factor => 1 pixel = 0.41 mm approx.
    %       height without baffle, but with alignment cube = 230 pixels =>
    %       94 mm 
    %       base width/length = 80 mm x 100 mm
    %      smallest sphere => 0.95 * sqrt(80^2 + 100^2 + 94^2) mm = 150 mm.
    %      Factor of 0.95 used to get a closer estimate to the true
    %      diameter of smallest enclosing sphere, since star tracker
    %      without baffle is not a cuboid.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        9.0     , 61.0 , 0.25 ,   nan   , 0.15;

    % VST-68M from Vectronic Aerospace GmbH
    %  Source: https://vectronic-aerospace.com/star-tracker-vst-68m/
    %  (see reference [VST68M]
    %  Note 1: Reading attitude errors from row "Accuracy 2sigma(x,y /z
    %          axis)" and converting 2-sigma values to 1-sigma values.
    %          Actual rms noise may be better, since quoted values might
    %          also include bias.
    %  Note 2: Assuming exposure time to be the inverse of output rate.
    %  Note 3: Estimates for the diameter of the smallest enclosing sphere
    %          without baffle:
    %     from the figure in this webpage
    %       height plus baffle = 675 pixels = 138 mm
    %        scale factor => 1 pixel = 0.20 mm approx.
    %       height without baffle = 293 pixels => 60 mm
    %       However, the VST-68M webpage states that its focal length is
    %       75 mm, therefore, its optics might protude inside the baffle.
    %       Assuming a height of 80 mm.
    %       base width/length = 60 mm x 60 mm
    %      smallest sphere => 0.95 * sqrt(60^2 + 60^2 + 80^2) mm = 110 mm.
    %      Factor of 0.95 used to get a closer estimate to the true
    %      diameter of smallest enclosing sphere, since star tracker
    %      without baffle is not a cuboid.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        2.5     , 15.0 , 0.2  ,   nan   , 0.11;

    % ST-16RT2 from Sinclair Interplanetary
    %  Sources: 
    %     file "star tracker RT 2019a.pdf" (Datasheet)
    %     file "ICD 1.29.pdf" (Interface Control Document Rev 1.29)
    %     (see reference [Sinclair])
    %  Note 1: Reading attitude errors from row "Absolute Accuracy" in
    %          the datasheet. Actual rms noise may be better, since
    %          quoted values also include bias introduced by thermal
    %          effects.
    %  Note 2: Assuming exposure time to be the inverse of output rate.
    %  Note 3: From the datasheet and ICD:
    %           dimensions without baffle: 62 mm x 56 mm x 37.05 mm
    %      smallest sphere => 0.95 * sqrt(62^2 + 56^2 + 37.05^2) mm =
    %                         = 86.8 mm.
    %      Factor of 0.95 used to get a closer estimate to the true
    %      diameter of smallest enclosing sphere, since star tracker
    %      without baffle is not a cuboid.

    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
        5.0     , 55.0 , 0.5  , 0.01023 , 0.0868 ;  % ST-16RT2
    
    % CubeStar
    %  Source: see reference [Erlank].
    %  Note 1: Reading attitude errors from Table 3, on page 9684 in
    %          reference [Erlank].
    %  Note 2: Assuming exposure time to be the inverse of the update rate.
    %  Note 3: dimensions: 46 mm x 33 mm x 70 mm
    %      smallest sphere => 0.95 * sqrt(46^2 + 33^2 + 70^2) mm =
    %                         85 mm.
    %      Factor of 0.95 used to get a closer estimate to the true
    %      diameter of smallest enclosing sphere, since star tracker
    %      without baffle is not a cuboid.
    %
    % yaw/pitch , roll , T_exp, aperture, STR_diam.
       36.0     , 108.0, 1.0  ,   nan   , 0.085 ; % CubeStar
    ];


% Computes the rms noise for a single optical head in radians, using the
% equation:
%    rms_error^2 = 2*rms_error_pitch_yaw^2 + rms_error_roll^2
% See Appendix A at the end of this file for a derivation of this equation.
STR_proc(:,1) = ...
    sqrt( 2*(STR_data(:,1) .^ 2) + (STR_data(:,2) .^ 2))  * arcsec;

% Computes combined parameter D^2 * t^(1/2) considering the smallest
% enclosing sphere diameter
STR_proc(:,2) = (STR_data(:,5) .^ 2) .* (STR_data(:,3) .^ 0.5);

% Computes combined parameter D^2 * t^(1/2) considering the optics
% aperture (NaNs used to mark unavailable fields.)
STR_proc(:,3) = (STR_data(:,4) .^ 2) .* (STR_data(:,3) .^ 0.5);

% Computes the rms noise for two identical optical heads in radians, using
% the equation, derived with the assumption that the two heads are mounted
% pointing to orthogonal directions to each other:
%    rms_error^2 = 0.5 * rms_error_pitch_yaw^2 + 
%                  2 / [rms_error_pitch_yaw^(-2) + rms_error_roll^(-2)]
% See Appendix B at the end of this file for a derivation of this equation.
STR_proc(:,4) = sqrt(...
    0.5 * (STR_data(:,1) .^ 2) + ...
    2 ./( (STR_data(:,1) .^ (-2)) + (STR_data(:,2) .^ (-2)) ) ...
    ) * arcsec;

% Computes combined parameter D^2 * t^(1/2) considering the approximate
% diameter of the smallest sphere that encloses two star tracker heads.
% Since an accurate determination of the diameter of this sphere would
% require a case by case analysis, here we adopt a more general approach
% that even though is not accurate will suffice for an initial analysis:
%  diameter_factor is the ratio between the diameters of the smallest
%  sphere that encloses two star tracker heads and the smallest sphere that
%  encloses only a single star tracker head. For two cubes side by side,
%  this factor is sqrt(6)/sqrt(3) = sqrt(2). For two cylinders side by
%  side, this factor is between 1.5 and 1.6. Choosing 1.6 to give some
%  allowance for cables and connectors.
diameter_factor = 1.6; 
STR_proc(:,5) = STR_proc(:,2) * diameter_factor^2;

% From the "Discussion and Results" section in the journal article:
%  Glow = lower estimate for constant G of equation (23).
%  Ghigh = higher estimate for constant G of equation (23).
Glow = sqrt(0.5)*1e-14; % unit: rad*m^2*s^(1/2)
Ghigh = 1e-13; % unit: rad*m^2*s^(1/2)

% Values of the theoretical limits estimates for the horizontal axis
% extremities.
pyini = Glow / pini;
pyend = Glow / pend;
pyini2 = Ghigh / pini;
pyend2 = Ghigh / pend;

% Minimum and maximum values in the horizontal axis.
xvalues = [STR_proc(:,2); pini; pend];
% xvalues = [STR_proc(:,2); STR_proc(:,3); pini; pend];
xmin = min(xvalues);
xmax = max(xvalues);

% Minimum and maximum values in the vertical axis in angleunits.
yvalues = [STR_proc(:,1); pyini; pyend; pyini2; pyend2];
ymin = min(yvalues) * botfactor / angleunit;
ymax = max(yvalues) * topfactor / angleunit;


% Diffraction and photon noise limited star sensors with limited FOV
% and limited to identifying stars up to J magnitude 12.575
% (brightest 26,036,431 2MASS stars in 2MASS J band).
% Data from calcTable2.m for D = 0.1m and t = 0.01s
strtable2 = [...
    % Consolidated list: theta_rms in radians:
    % FOV(deg)	head 1	  head 2	both heads
         360	3.51675e-11	3.51675e-11	2.48672e-11
          90	1.78909e-10	9.77819e-11	7.37818e-11
          25	3.10873e-09	5.81849e-10	3.65386e-10
          10	1.59588e-08	5.96812e-09	1.03166e-09
           4	1.53098e-07	5.21986e-08	3.4922e-09
    ];

% Computing the D^2 * t^(1/2) metric for the star sensors above
str2_D = 0.1; % diameter in meters
str2_t = 0.01; % exposure time in seconds
%  Single head
str2_single_D2t = str2_D^2 * sqrt(str2_t);
%  Two heads - assuming the diameter of the smallest enclosing sphere to
%              be twice the diameter of a single optical head.
str2_dual_D2t = str2_single_D2t * 2^2;

%%% Plot Generation and legend labels preparation
% Plotting star sensors with a log-log plot:
fig1 = figure;
hplots = zeros(1,11);
aa = 1/angleunit;
newline = char(10); % new line character

%  Plotting real examples of single head star sensors, using the estimated
%  diameter of the minimum enclosing sphere:
hplots(1) = loglog(STR_proc(:,2), STR_proc(:,1) * aa, 'o'); 
hold on; % this command enables additional plots on the same figure.
legendlabels{1} = [newline 'single head' newline ...
    'star sensor,' newline 'alone'];

%  Plotting combinations two equal star sensors pointing at orthogonal
%  directions:
hplots(2) = loglog(STR_proc(:,5), STR_proc(:,4) * aa, 's');
legendlabels{2} = [newline 'two identical' newline ...
    'star sensors' newline 'in combination'];

%  Plotting real examples of single head star sensors, using their optics
%  aperture diameter, when available. (entries with NaNs are ignored)
hplots(3) = loglog(STR_proc(:,3), STR_proc(:,1) * aa, '*');
legendlabels{3} = [newline 'single, D^2t^{1/2}' newline ...
    'calculated' newline 'from aperture' newline 'diameter'];

if isoctave() == true
    % In Octave 5.1.0 for Windows, we have to replace some legend labels
    % with a shorter version, to make them fit.
    legendlabels{1} = [newline 'single head star' newline ...
        'sensor, alone'];
    legendlabels{2} = [newline 'two identical' newline 'star sensors'];
    legendlabels{3} = [newline 'calculated from' newline 'aperture diam.'];
end

grid on;


% Symbol for vartheta_{rms,min} in legend labels:
varthetasymb = '\vartheta_{rms,min}';
if isoctave() == true % if running Octave
    % Octave 5.1.0 for Windows does not support \vartheta, therefore
    % replacing it with a regular \theta - MAAF 2019.05.22.
    varthetasymb = '\theta_{rms,min}';
end

% Adding curves with upper and lower estimates of the theoretical limits on
% attitude accuracy:
%  Upper curve:
hplots(4) = loglog([pini; pend], [pyini2; pyend2] * aa, 'k--', ...
    'LineWidth', 1.0);
legendlabels{4} = [newline ' upper estimate' newline ...
    ' of  ' varthetasymb];

%  Lower curves:
hplots(5) = loglog([pini; pend], [pyini; pyend]   * aa, 'k-', ...
    'LineWidth', 1.0);
legendlabels{5} = [newline ' lower estimate' newline ...
    ' of  ' varthetasymb];

% Adding "ideal" star sensors with limited field of view and limited star
% catalog:
np_prev = size(legendlabels,2);
numcombs = size(strtable2, 1);
strtable2_D2t = [str2_single_D2t, str2_single_D2t, str2_dual_D2t]';
markerlist = ['x', '^', '>', '<', 'v']; % list of markers
markerlistsz = size(markerlist,2);

%  Degree sign for legend labels.
if isoctave() == false % Matlab R2015b (tested on Windows)
    degree_sign = char(176);
else % Octave
    degree_sign = '\circ';
end

%  Do the inclusion of 'ideal' star sensors with limited FOV and mag.:
np_prev2 = np_prev + numcombs; 
for k = 1:numcombs
    idx = np_prev2 + 1 - k;
    hplots(idx) = ...
        plot(strtable2_D2t, strtable2(k, 2:4)' * aa,...
        markerlist(mod(k - 1, markerlistsz) + 1));        
    legendlabels{idx} = ...
        [ '\0_{FOV} = ' num2str(strtable2(k,1)) degree_sign];
end

% Draws lines in light colors joining star, circle and square markers
% belonging to the same star sensors:
n_STR = size(STR_proc, 1);
if isoctave() == false  % Matlab R2015b
    % Setting the z-coordinates of lines to be plotted to negative values,
    % places them behind other picture elements.
    zz_joininglines = [-1, -1];
else % Octave 5.1.0
    % Setting the z-coordinates of lines to be plotted to negative values
    % caused issues with other picture elements in Octave v. 5.1.0.
    zz_joininglines = [0, 0];
end
    
for k=1:n_STR
    % For each star sensor, draw a line connecting:
    %       one head      two heads
    line( [STR_proc(k,2), STR_proc(k,5)], ...      % x values (horiz.)
          [STR_proc(k,1), STR_proc(k,4)] * aa, ... % y values (vert.)
          zz_joininglines, ... % draw behind the markers in Matlab.
          'LineWidth', 0.5, 'Color', [0.5 0.95 0.5]);
      
    % For each star sensor, draw a line connecting:
    %      apert. diam      one head
    line( [STR_proc(k,3), STR_proc(k,2)], ...      % x values (horiz.)
          [STR_proc(k,1), STR_proc(k,1)] * aa, ... % y values (vert.)
          zz_joininglines, ... % draw behind the markers in Matlab.
          'LineWidth', 0.5, 'Color', [0.5 0.75 1], ...
          'LineStyle', '--');      
end


% Adjust plot scale
axis([xmin xmax ymin ymax]);

% Adjust plot size
set(gcf, 'Units', 'inches');
set(gcf, 'Position', [screen_posx screen_posy  plotwidth plotheight]);
set(gcf, 'PaperSize', [plotwidth plotheight]);
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperPosition', [0, 0, plotwidth, plotheight]);

% Replaces y-axis ticks
ytickA = ceil(log10(ymin)); % log10 of first vertical tick value
ytickB = floor(log10(ymax)); % log10 of last vertical tick value
yticks = 10 .^ linspace(ytickA, ytickB, ytickB - ytickA + 1);
set(gca, 'YTick', yticks);

% Replaces x-axis ticks
xtickA = ceil(log10(xmin)); % log10 of first horizontal tick value
xtickB = floor(log10(xmax)); % log10 of last horizontal tick value
xticks = 10 .^ linspace(xtickA, xtickB, xtickB - xtickA + 1);
set(gca, 'XTick', xticks);

% Labels
xlabel('Combined diameter-exp. time metric D^{2}t^{1/2} / [m^2s^{1/2}]');
if angleunit == arcsec
    ylabel('Attitude error rms value in arc seconds');
elseif angleunit == arcmin    
    ylabel('Attitude error rms value in arc minutes');
elseif angleunit == deg
    ylabel('Attitude error rms value in degrees');
elseif angleunit == 1
    ylabel('Attitude error rms value in radians');
else
    ylabel_text = sprintf('Attitude error rms value / %g rad', angleunit);
    ylabel(ylabel_text);
end

% In Matlab R2015b, uncomment this line below to be able to pan the plot in
% the plot window. Side effects: no labels, no .pdf figure.
% return;


axes1 = gca;

if isoctave() == false % Matlab R2015b 
    set(axes1, 'Position',[0.09 0.09 0.665 0.90]);
else % Octave
    set(axes1, 'Position',[0.09 0.10 0.665 0.89]);
end

aL1 = axes('visible', 'off');
set(aL1, 'Position', [0 0 1 1]);
axis([0 1 0 1]);

% Legend
if isoctave() == false % Matlab R2015b
    text(0.77, 0.98, 'Key:');
    hL1 = legend(aL1, hplots(1:np_prev), legendlabels(1:np_prev), ...
        'Location', [0.87 0.70 0.01 0.01], 'Box', 'off');
else % Octave
    % In Octave, creating two legends didn't work. Therefore we
    % create a single legend here with all entries
    hL1 = legend(aL1, hplots(1:np_prev2), legendlabels, 'Box', 'off');    
end

aL2 = axes('visible', 'off'); hold on;
set(aL2, 'Position', [0 0 1 1]);
if isoctave() == false % Matlab R2015b
    text(0.78, 0.36, 'Magnitude limited');
    text(0.78, 0.33, 'ideal star sensors');
    text(0.78, 0.30, 'at m_J = 12.575, with:');
    
    leg2location = [0.87, 0.27 - numcombs * 0.02, 0.01, 0.01];
    hL2 = legend(aL2, hplots(np_prev + 1: np_prev2), ...
        legendlabels(np_prev + 1: np_prev2), ...
        'Location', leg2location, 'Box', 'off');
else  % Octave
    text(0.78, 0.14, 'Triangular and cross');
    text(0.78, 0.11, 'markers indicate FOV');    
    text(0.78, 0.08, 'and magnitude limited');
    text(0.78, 0.05, 'at m_J = 12.575 ideal');
    text(0.78, 0.02, 'star sensors.');
end

% Draws three ellipses on the plot marking the regions with real
% examples of star sensors, and: single head and dual headed ideal star
% sensors limited at J magnitude 12.575:
axis([0 1 0 1]);
tt = linspace(0, 2*pi);
ellipse_color = [1 0.5 0.5];
text_color = [0.4, 0.4, 0.4];

% first ellipse
plot(0.425 + 0.325 * sign(cos(tt)).*abs(cos(tt)).^(0.5),...
    0.87 + 0.11 * sign(sin(tt)).*abs(sin(tt)).^(0.5), ...
    'LineWidth', 0.5, 'Color', ellipse_color);
text(0.22, 0.96, 'real star sensors', ...
    'Color', text_color);

% second ellipse
plot(0.33 + 0.035 * cos(tt), 0.51 + 0.18 * sin(tt), '--', ...
    'LineWidth', 0.5, 'Color', ellipse_color);
text(0.18, 0.2, [...
    'ideal, single' newline ...
    'optical head'],...
    'Color', text_color);
line ([0.25, 0.33], [0.23, 0.33], ...
    'LineStyle', '--', 'LineWidth', 0.5, 'Color', ellipse_color);


% third ellipse
plot(0.44 + 0.03 * cos(tt), 0.43 + 0.115 * sin(tt), '--', ...
    'LineWidth', 0.5, 'Color', ellipse_color);
text(0.35, 0.17, [...
    '  ideal, two' newline ...
    'optical heads'],...
    'Color', text_color);
line ([0.415, 0.44], [0.20, 0.315], ...
    'LineStyle', '--', 'LineWidth', 0.5, 'Color', ellipse_color);

% Save plot as an image file
print(gcf, 'outputs/STRcomp', '-dpdf');
%print(gcf, 'outputs/STRcomp', '-dpng', '-r300');


% References: (also present in the research article)
%
%  [Astro15]   Jenaoptronik, Autonomous Star Sensor ASTRO 15, 2015.
%              https://www.jena-optronik.de/en/aocs/astro15.html
%              (accessed on 11 Nov. 2019).
%
%  [Terma2019] TERMA A/S Space, T1 $ T2 Star Trackers. Miniaturized
%              Optical Heads and Electronic Unit, 2019.
%              https://www.terma.com/space/space-segment/star-trackers
%              (accessed on 8 Nov. 2019).
%
%  [Auriga]    Sodern, Auriga - Last Innovative Star Tracker, 2017. 
%              http://www.sodern.com/website/en/ref/Auriga_315.html
%              (accessed on 11 Nov. 2019).
%
%  [Leonardo}  Leonardo - Societa per azioni, Autonomous Star Trackers, 2017
%              http://www.leonardocompany.com/en/-/aastr
%              (first access on 24 May 2017, second access on 11 Nov. 2019)
%
%  [VST41M]    Vectronic Aerospace GmbH, Star Tracker VST-41M.
%              https://vectronic-aerospace.com/star-sensor-vst-41m
%              (accessed on 11 Nov. 2019).
%
%  [VST68M]    Vectronic Aerospace GmbH, Star Tracker VST-68M.
%              https://vectronic-aerospace.com/star-tracker-vst-68m/
%              (accessed on 11 Nov. 2019).
%
%  [Sinclair]  Sinclair Interplanetary, Star Trackers, 2019.
%              http://www.sinclairinterplanetary.com/startrackers
%              (accessed on 12 Nov. 2019).
%
%  [Erlank]    Erlank, A. O. and Steyn, W. H. Arcminute Attitude 
%              Estimation for CubeSats with a Novel Nano Star Tracker.
%              IFAC, Vol. 47, 2014, pp. 96799684.
%



% Appendix A - Derivation of the equations used to compute the rms noise
%              for a single head star sensor used in isolation.
%
%      The covariance matrix of the attitude error of a star sensor, will
%   depend on the direction in the sky where it is pointed to (that is, in
%   the number and brightness of the stars that are in its field of
%   view), among other factors. However, to know this covariance matrix
%   exactly would require a very detailed model of the star sensor, which
%   probably only the manufacturer has. Therefore, we will base our
%   analysis on attitude error information made public by the
%   manufacturers, which is expected to be worse than the typical
%   performance of the star sensor in a real condition.
%      With this consideration, it is reasonable to assume that its
%   covariance matrix P_theta_theta is:
%                     
%                      ( sigma_yawpitch^2       ??                ??  )
%     P_theta_theta =  (       ??        sigma_yawpitch^2         ??  )
%                      (       ??               ??       sigma_roll^2 )
%
%   when we take X and Y as being cross-boresight axes and Z as the
%   boresight axis.
%
%   The trace of this matrix will be:
%
%      tr(P_theta_theta) = 2 * sigma_yawpitch^2 + sigma_roll^2,
%
%   which is rms_error^2 (comment at line 266 in this file, if line 268
%   reads "STR_proc(:,1) = ..."), according to equation (22) in the journal
%   article, that is, the square of the rms noise for a single head star
%   sensor used in isolation.
%
%   The off-diagonal terms of P_theta_theta (cross-correlation terms),
%   marked with ?? don't matter, as they are not used.



% Appendix B - Derivation of the equations used to compute the rms noise
%              for two equal star sensors used in combination, when they
%              point at orthogonal directions.
%
%   For this derivation, we will assume that one star sensor (let us call
%   it STR1) will be pointing towards the X direction and the other (let us
%   call it STR2) will be pointing towards the Y direction, being X, Y and
%   Z orthogonal axes of a reference in the spacecraft were they are
%   mounted. (We are assuming they are rigidly mounted on the same
%   spacecraft, and pointing in directions perpendicular to each other. For
%   simplicity, we also assume this spacecraft has only these two star
%   trackers as attitude sensors.)
%
%   For any given star sensor attitude, the off-diagonal terms of the
%   attitude error covariance matrix will usually be non-zero, due to
%   asymmetries in the distribution of stars in its field of view. However,
%   if we consider an average covariance matrix for all possible attitudes
%   together, these asymmetries tend to cancel each other, leaving to
%   almost zero off-diagonal terms in the covariance matrix. In other
%   words, taken all possible star sensor attitudes together, the errors
%   along the optical axis and the cross-boresight axes become
%   uncorrelated. As we are considering an 'average' for all possible
%   attitudes, we have the following covariance matrices for STR1 and STR2:
%
%   For STR1:
%                       ( sigma_roll^2        0                    0  )
%     P1_theta_theta =  (        0     sigma_yawpitch^2            0  )
%                       (        0            0      sigma_yawpitch^2 )
%
%   For STR2:
%                       ( sigma_yawpitch^2     0                   0  )
%     P2_theta_theta =  (        0        sigma_roll^2             0  )
%                       (        0             0     sigma_yawpitch^2 )
%
%   If we combine their measurements using an optimal estimator, each star
%   sensor will contribute to the spacecraft attitude knowledge with their
%   information matrices, which are the inverses of their measurement error
%   covariance matrices, therefore:
%
%      F_spacecraft = F_STR1 + F_STR2,   with
%
%      F_STRi = inv(Pi_theta_theta),  i = 1,2;
%
%      inv(M) = inverse of matrix M.
%
%   F_spacecraft is the attitude knowledge information matrix for the
%   spacecraft. F_STR1 and F_STR2 are the attitude knowledge matrices for
%   STR1 and STR2, respectively.
%   
%   From previous equations, we have:
%   For STR1:
%               ( 1/sigma_roll^2           0                     0  )
%     F_STR1 =  (        0       1/sigma_yawpitch^2              0  )
%               (        0                 0     1/sigma_yawpitch^2 )
%
%   For STR2:
%               ( 1/sigma_yawpitch^2     0                      0  )
%     F_STR2 =  (        0        1/sigma_roll^2                0  )
%               (        0               0      1/sigma_yawpitch^2 )
%
%
%  Therefore:
%     F_spacecraft(1,1) = 1/sigma_roll^2 + 1/sigma_yawpitch^2;
%     F_spacecraft(2,2) = 1/sigma_roll^2 + 1/sigma_yawpitch^2;
%     F_spacecraft(3,3) = 2/sigma_yawpitch;
%     F_spacecraft(i,j) = 0 for i and j different.
%
%  The spacecraft covariance matrix will be the inverse of its attitude
%  knowledge matrix. Its main diagonal components will be:
%     P_theta_theta_spacecraft(1,1) = 1 / F_spacecraft(1,1)
%     P_theta_theta_spacecraft(2,2) = 1 / F_spacecraft(2,2)
%     P_theta_theta_spacecraft(3,3) = 0.5 * sigma_yawpitch^2.
%
%  Taking the trace of P_theta_theta_spacecraft (eq. 22 in the paper), we
%  obtain:
%
%     rms_error_spacecraft^2 = 2 / (1/sigma_roll^2 + 1/sigma_yawpitch^2) +
%                              0.5 * sigma_yawpitch^2
%
%  which is the same equation written at lines 282-283 of this file
%  (assuming the line 285 reads "STR_proc(:,4) = sqrt(..."), written in a
%  different form.
%


% The end.