function [starcat, rejrows] = readHipparcos(filename, numlines, rejbad)
% READHIPPARCOS - Reads the Hipparcos catalog.
%
% [starcat, rejrows] = readHipparcos(filename, numlines, rejbad) reads
% the Hipparcos catalog, saving into matrix starcat all the entries
% in the Hipparcos catalog with valid right ascension, declination, 
% V magnitudes, B-V color indexes and V-I color indexes. Entries with 
% invalid fields may be preserved in the output, depending on the value
% of rejbad parameter.
%
% Inputs (optional):
%  filename = file name of Hipparcos catalog. Defaults to 'hip_main.dat'
%  numlines = number of lines in the catalog to process.
%             Defaults to 118218.
%  rejbad = if true, rejects catalog entries with one or more invalid or
%           blank fields of those read. If false, these entries are kept
%           in the result, but with invalid fields marked with NaNs
%           (Not a Number).
%           Default: true => reject all entries with missing/bad fields.
%
% Outputs:
%  starcat: an n x 6 matrix, with:
%   first column => star identifier in the Hipparcos catalog
%   second column => star right ascension in degrees
%   third column => star declination in degrees
%   fourth column => Johnson's V magnitude
%   fifth column => Johnson's B-V color index
%   sixth column => Johnson's V-I color index
%
%  rejrows:
%   an array listing the lines in source file that were rejected.
%
% Copyright (C) 2016-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

t1 = tic;
warnflag = false; % indicates the occurrence of warnings.

if nargin < 1
    filename = 'hip_main.dat'; % Hipparcos catalog
    % filename = 'c:\ptase\hipparcos\CAT_M5.TXT';
        % CAT_M5.txt is a subset of Hipparcos catalog used for testing
end

if nargin < 2
    numlines = 118218; % number of entries in the Hipparcos catalog
    % numlines = 1648; % number of entries in CAT_M5.TXT catalog subset.
end

if nargin < 3
    rejbad = true; % Rejects all entries with missing or bad fields
end

fid = fopen(filename);
if fid == -1
    error (['Couldn''t open file "' filename '".']);
end

validf = false(numlines, 1);
datain = zeros(numlines, 6);

% Counters for statistical purposes:
acceptcnt = 0; % number of lines=entries to be copied to the output matrix
malformed_line_cnt = 0;  % comment lines or mal-formed lines
good_star_cnt = 0; % count of entries in the catalog with no issues
                   % encountered during reading (good stars).
stars_bad_ID = 0;  % stars with bad or blank catalog number (field H1)
stars_bad_RA = 0;  % stars with bad or blank right ascension (field H8)
stars_bad_dec = 0; % stars with bad or blank declination (field H9)
stars_bad_mag = 0; % stars with bad or blank V magnitude (field H5)
stars_bad_BV = 0;  % stars with bad or blank B-V color index (field H37)
stars_bad_VI = 0;  % stars with bad or blank V-I color index (field H40)
rejcount = 0; % count of rejected lines.

unexpected_file_end = false;

tline = fgets(fid);
for n=1:numlines

    % Checks for unexpected early file termination
    if ~ischar(tline)
        unexpected_file_end = true;
        break;
    end
    
    rejectline = false; % true when current line should be rejected.
    
    % Protection against incomplete lines
    if size(tline, 2) < 265
        rejectline = true;
        malformed_line_cnt = malformed_line_cnt + 1;
    else
        % Protection against comment lines/etc:
        % Checks for field delimiters in the Hipparcos catalog.
        if (    (tline(2)   ~= '|') || (tline(15)  ~= '|') || ...
                (tline(41)  ~= '|') || (tline(47)  ~= '|') || ...
                (tline(51)  ~= '|') || (tline(64)  ~= '|') || ...
                (tline(77)  ~= '|') || (tline(245) ~= '|') || ...
                (tline(260) ~= '|') || (tline(265) ~= '|'))
            rejectline = true;
            malformed_line_cnt = malformed_line_cnt + 1;
        end
    end
        
    % Reads fields H1, H8, H9, H5, H37 and H40 from Hipparcos catalog
    % Reference: Volume 1, Part 2, section 2.1 of the Hipparcos
    %            Catalogue manual, pages 103-138. ISSN 0379-6566
    %            ISBN 929092399-7. Available for download from:
    %            http://www.cosmos.esa.int/web/hipparcos/catalogues
    if rejectline == false
        
        % goodflag indicates whether there was a problem reading 
        % a Hipparcos field for current line
        goodflag = true; % true means no problem detected
        
        % Reads star-ID (field H1)
        [starID, badread] = string2double(tline(3:14));
        if badread || not(isfinite(starID))
            stars_bad_ID = stars_bad_ID + 1;
            goodflag = false;
            starID = nan;
        end
        
        % Reads right ascension (field H8)
        [RA, badread] = string2double(tline(52:63));
        if badread || not(isfinite(RA))
            stars_bad_RA = stars_bad_RA + 1;
            goodflag = false;
            RA = nan;
        end
        
        % Reads declination (field H9)
        [dec, badread] = string2double(tline(65:76));
        if badread || not(isfinite(dec))
            stars_bad_dec = stars_bad_dec + 1;
            goodflag = false;
            dec = nan;
        end
        
        % Reads magnitude in Johnson's V band (field H5)
        [magV, badread] = string2double(tline(42:46));
        if badread || not(isfinite(magV))
            stars_bad_mag = stars_bad_mag + 1;
            goodflag = false;
            magV = nan;
        end
        
        % Reads B-V color index (field H37)
        [BV, badread] = string2double(tline(246:251));
        if badread || not(isfinite(BV))
            stars_bad_BV = stars_bad_BV + 1;
            goodflag = false;
            BV = nan;
        end
        
        % Reads V-I color index (field H40)
        [VI, badread] = string2double(tline(261:264));
        if badread || not(isfinite(VI))
            stars_bad_VI = stars_bad_VI + 1;
            goodflag = false;
            VI = nan;
        end
        
        datain(n, 1) = starID;
        datain(n, 2) = RA;
        datain(n, 3) = dec;
        datain(n, 4) = magV;
        datain(n, 5) = BV;
        datain(n, 6) = VI;
        
        if goodflag
            good_star_cnt = good_star_cnt + 1;
        else
            if rejbad % if entries with issues should be rejected:
                rejectline = true;
            end
        end
    end
    
    if rejectline
        rejcount = rejcount + 1;
        validf(n) = false; % entry should be discarded
    else
        acceptcnt = acceptcnt + 1;
        validf(n) = true; % entry should be copied to final output
    end

    % Reads next line
    tline = fgets(fid);
end

fclose(fid);

% If an unexpected early file termination has occurred
if unexpected_file_end
    n = n-1;
    warning(['End of file reached at line %d.\n' ...
        'Either source catalog is truncated or numlines is set to'...
        ' a value larger than the catalog size.  numlines=%d.'],...
        n, numlines);
    warnflag = true;
end

% If file end still has not being reached and input buffer is full
if ischar(tline)
    warning(['End of file still not reached after processing '...
        'numlines=%d lines.\n'...
        'Some stars from the source catalog might have been ignored.\n'...
        'Try to set parameter numlines to a larger value '...
        'and try again.'], numlines);
    warnflag = true;
end

% Copies only valid entries to the output
starcat = datain(validf,:);

% Creates a list of rejected lines in the input file.
idxlist = 1:n;
rejrows = idxlist(~validf(1:n));

% Print some messages.
fprintf('Read %d lines from file %s.\n', n, filename);
fprintf('Of these:\n');
fprintf(' %6d lines accepted and converted to stars;\n', acceptcnt);
fprintf(' %6d lines rejected.\n', rejcount);
fprintf('Details:\n');
fprintf(' %6d lines with no issues (good stars);\n', good_star_cnt);
fprintf(' %6d entries with bad catalog number;\n', stars_bad_ID);
fprintf(' %6d entries with bad right ascension;\n', stars_bad_RA);
fprintf(' %6d entries with bad declination;\n', stars_bad_dec);
fprintf(' %6d entries with bad V magnitude;\n', stars_bad_mag);
fprintf(' %6d entries with bad B-V color index;\n', stars_bad_BV);
fprintf(' %6d entries with bad V-I color index;\n', stars_bad_VI);
fprintf(' %6d comment or malformed lines.\n', malformed_line_cnt);
if rejbad
    fprintf('Bad entries rejected.\n');
else
    fprintf('Bad entries included in the ouput.\n');
end

if warnflag
    warning('One or more warnings occurred. Output may be compromised!');
end

dt1 = toc(t1);
fprintf('Catalog read in %f seconds.\n', dt1);

end