function [starcat, badrows, sectrows] = readTycho2(filename, rejbad,...
     manysect, numlines)
% READTYCHO2
% [starcat, badrows, sectrows] = readTycho2(filename, rejbad,...
%      manysect, numlines)
% Reads the Tycho-2 star catalog, saving into matrix starcat all the
% entries in the catalog with valid right ascension, declination, 
% VT and BT magnitudes. Entries with invalid fields may be preserved
% in the output, depending on the value of rejbad parameter.
%
% Inputs (optional):
%  filename = file name or base filename of Tycho2 star catalog.
%             Defaults to 'tyc2.dat'
%  rejbad = if true, rejects catalog entries with one or more invalid or
%           blank fields of those read. If false, these entries are kept
%           in the result, but with invalid fields marked with NaNs
%           (Not a Number).
%           Default: true => reject all entries with missing/bad fields.
%  manysect = A flag indicating whether the Tycho2 star catalog is split in
%             multiple sections or not. If manysect is true, it is assumed
%             that the Tycho2 star catalog is split in 20 sections, each
%             having a name starting with the name supplied in filename
%             plus a dot and a number from 0 to 19 specifiying the section.
%             For example, if filename == 'tyc2.dat' then this function
%             will read from 'tyc2.dat.00' to 'tyc2.dat.19'. If manysect is
%             false, then this function will read the catalog from a single
%             file, specified in filename.
%             Defaults to true.
%  numlines = number of lines in the catalog to process.
%             Defaults to 2,539,913.
%
% Outputs:
%  starcat: an n x 5 matrix, with:
%   first column => position of star in the Tycho-2 catalog (row number)
%   second column => star right ascension in degrees
%   third column => star declination in degrees
%   fourth column => Tycho BT magnitude
%   fifth column => Tycho VT magnitude
%
%  badrows:
%   an array listing the lines in source file(s) that were rejected.
%   The lines are counted as if all sections were merged in a single file
%
%  sectrows:
%   an array listing the number of lines found in each catalog section.
%
% Copyright (C) 2017-2019 INPE - Instituto Nacional de Pesquisas Espaciais.
% This is free software - see file LICENSE.txt for terms and conditions.

% Code written by Marcio A. A. Fialho.

t1 = tic;
warnflag = false; % indicates the occurrence of warnings.

% Default parameters
if nargin < 1
    filename = 'tyc2.dat'; % Tycho star catalog
end

if nargin < 2
    rejbad = true; % Rejects all entries with missing or bad fields
end

if nargin < 3
    manysect = true;
end

if nargin < 4
    numlines = 2539913; % number of entries in tyc2.dat
end

validf = false(numlines, 1);
datain = zeros(numlines, 5);

% Counters for statistical purposes:
acceptcnt = 0; % number of lines=entries to be copied to the output matrix
malformed_line_cnt = 0;  % comment lines or mal-formed lines
good_star_cnt = 0; % count of entries in the catalog with no issues
                   % encountered during reading (good stars).
stars_bad_ID = 0;  % stars with bad or blank catalog number (colums 1-12)
stars_bad_RA = 0;  % stars with bad or blank right ascension (column 153-164) 
stars_bad_dec = 0; % stars with bad or blank declination (column 166-178)
stars_bad_BT = 0;  % stars with bad or blank BT magnitude (column 111-116)
stars_bad_VT = 0;  % stars with bad or blank VT magnitude(column 124-129)
rejcount = 0; % count of rejected lines.

if manysect
    numsections = 20;
    
    % Tests whether there exists a file beyond the 20-th section
    % If it exists, then the Tycho2 catalog might have been split in more
    % than 20 sections.  Remember: sections are numbered from 0 to 19.
    filename_sect_beyond = sprintf('%s.%.2d', filename, numsections); 
    if exist(filename_sect_beyond, 'file')
        warning(['File %s exists. This means the catalog is probably '...
            'split in more than %d sections, but only first %d are '...
            'being considered.'], filename_sect_beyond, numsections,...
            numsections); %#ok<SPWRN>
    end
else
    numsections = 1;
    filename_sect = filename;  % 'tyc2.dat'
end

sectrows = zeros(numsections, 1);

nsect = 1; % current catalog section
n = 0; % total number of lines read

while nsect <= numsections
        
    if n > numlines
        break;
    end
    
    if manysect
        % filename_sect = 'tyc2.dat.00' to 'tyc2.dat.19'
        filename_sect = sprintf('%s.%.2d', filename, nsect-1);
        fprintf('Reading section %d/%d from file %s\n',...
            nsect, numsections, filename_sect);
    else
        fprintf('Reading catalog from file %s\n', filename_sect);
    end

    fid = fopen(filename_sect);
    if fid == -1
        error (['Couldn''t open file "' filename_sect '".']);
    end
    
    tline = fgets(fid);
    linenum = 0;
    while true
    
        % Checks if line was successfully read
        if ischar(tline)
            linenum = linenum + 1;
            n = n + 1;
            if n > numlines
                break;
            end
        else
            % Line is empty - End of File reached
            sectrows(nsect) = linenum;
            nsect = nsect + 1; % selects the next section
            break;
        end
        
        rejectline = false; % true when current line should be rejected.
        
        % Protection against incomplete lines
        if size(tline, 2) < 206
            rejectline = true;
            malformed_line_cnt = malformed_line_cnt + 1;
        else
            % Protection against comment lines/etc:
            % Checks for field delimiters in the Tycho2 catalog.
            if (    (tline(5)   ~= ' ') || (tline(11)  ~= ' ') ||...
                    (tline(13)  ~= '|') || ...
                    (tline(110) ~= '|') || (tline(117) ~= '|') || ...
                    (tline(123) ~= '|') || (tline(130) ~= '|') || ...
                    (tline(152) ~= '|') || (tline(165) ~= '|') || ...
                    (tline(178) ~= '|') )
                rejectline = true;
                malformed_line_cnt = malformed_line_cnt + 1;
            end
        end
            
        % Reads Tycho-2 fields
        % Reference: E. Hog, C. Fabricus, V.V. Makarov. "Guide to the
        % Tycho-2 Catalogue." 14 Jan 2000.
        if rejectline == false
            
            % goodflag indicates whether there was a problem reading 
            % a Hipparcos field for current line
            goodflag = true; % true means no problem detected
                   
            % Reads BT magnitude
            [BT, badread] = string2double(tline(111:116));
            if badread || not(isfinite(BT))
                stars_bad_BT = stars_bad_BT + 1;
                goodflag = false;
                BT = nan;
                % fprintf('n=%d => BT = %s\n',n, [tline(111:116) 0]);
            end
            
            % Reads VT magnitude
            [VT, badread] = string2double(tline(124:129));
            if badread || not(isfinite(VT))
                stars_bad_VT = stars_bad_VT + 1;
                goodflag = false;
                VT = nan;
                % fprintf('n=%d => VT = %s\n',n, [tline(124:129) 0]);
            end
            
            % Reads observed Tycho-2 right ascension
            [RA, badread] = string2double(tline(153:164));
            if badread || not(isfinite(RA))
                stars_bad_RA = stars_bad_RA + 1;
                goodflag = false;
                RA = nan;
            end
            
            % Reads observed Tycho-2 declination
            [dec, badread] = string2double(tline(166:177));
            if badread || not(isfinite(dec))
                stars_bad_dec = stars_bad_dec + 1;
                goodflag = false;
                dec = nan;
            end
            
            datain(n, 1) = n;
            datain(n, 2) = RA;
            datain(n, 3) = dec;
            datain(n, 4) = BT;
            datain(n, 5) = VT;
            
            if goodflag
                good_star_cnt = good_star_cnt + 1;
            else
                if rejbad % if entries with issues should be rejected:
                    rejectline = true;
                end
            end
        end
        
        if rejectline
            rejcount = rejcount + 1;
            validf(n) = false; % entry should be discarded
        else
            acceptcnt = acceptcnt + 1;
            validf(n) = true; % entry should be copied to final output
        end
    
        % Reads next line
        tline = fgets(fid);
    end
    
    fclose(fid);
end


% If an unexpected early file termination has occurred
if n < numlines
    warning(['End of file reached in file %s at line %d.\n' ...
        'Either source catalog is truncated or numlines is set to'...
        ' a value larger than the catalog size or catalog is split'...
        ' in more than %d sections.  numlines=%d'],...
        filename_sect, linenum, numsections, numlines);
    warnflag = true;
end

% If file end still has not being reached and input buffer is full
if n > numlines
    warning(['End of last section still not reached after '...
        'processing numlines=%d lines in %d sections.\n'...
        'Some stars from the source catalog might have been ignored.\n'...
        'Try to set parameter numlines to a larger value '...
        'and try again'], numlines, nsect);
    warnflag = true;
end

% Copies only valid entries to the output
fprintf('Generating output star catalog.\n');
starcat = datain(validf,:);

% Creates a list of rejected lines in the input file.
nx = min(n, numlines);
idxlist = 1:nx;
badrows = idxlist(~validf(1:nx));

% Print some messages.
if manysect
    fprintf('Read %d lines from files %s.00 to %s\n',...
        nx, filename, filename_sect);
else
    fprintf('Read %d lines from file %s\n', nx, filename);
end
fprintf('Of these:\n');
fprintf(' %d lines accepted and converted to stars\n', acceptcnt);
fprintf(' %d lines rejected\n', rejcount);
fprintf('Details:\n');
fprintf(' %d lines with no issues (good stars)\n', good_star_cnt);
fprintf(' %d entries with bad catalog number\n', stars_bad_ID);
fprintf(' %d entries with bad right ascension\n', stars_bad_RA);
fprintf(' %d entries with bad declination\n', stars_bad_dec);
fprintf(' %d entries with bad BT magnitude\n', stars_bad_BT);
fprintf(' %d entries with bad VT magnitude\n', stars_bad_VT);
fprintf(' %d comment or malformed lines\n', malformed_line_cnt);

if rejbad
    fprintf('Bad entries were rejected.\n');
else
    fprintf('Bad entries were included in the ouput.\n');
end

if warnflag
    warning('One or more warnings occurred. Output may be compromised!');
end

dt1 = toc(t1);
fprintf('Catalog read in %f seconds.\n', dt1);

end